
#include "stdafx.h"
#include "sdlacfg.h"


HKEY	hkey = 0;
GLOBALS Globals_Old_Cfg;
GLOBALS Globals_New_Cfg;
BOOL	okay = FALSE;

//driver cfg functions prototypes
BOOL DriverSaveCfgData(HWND hdlg);

int 
DriverOnNotify(
	HWND    ParentHwnd,
	LPNMHDR NmHdr,
	SETUPSTUFF * Params
	);
void DriverInit(HKEY hkey, HWND hdlg);

static LRESULT CALLBACK RestartingDriverDlg(HWND hdlg, UINT iMessage, WPARAM wParam, LPARAM lParam);

BOOL WINAPI DrvPageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	switch (msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DRV_CFG_PAGE, "Drv: WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if(DriverOnNotify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DRV_CFG_PAGE, "Invalid Driver Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:	//this message received only ONCE
		{						
			Debug(DRV_CFG_PAGE, "Drv:WM_INITDIALOG");

			// Open registry for our device
			// This is done in device cfg page since it is the first page to open.
			if (okay == TRUE)
			{	
				//using configuration from the registry, update visual controls
				DriverInit(hkey, hdlg);
			}

			if (!okay)
			{	// disable everything
				EnableWindow( GetDlgItem(hdlg, IDCHECK_IS_LOG_TO_FILE), FALSE);
			}
		}
		break;
		
   	case WM_DESTROY:
		{
			LPSTR infofile = (LPSTR) GetWindowLong(hdlg, DWL_USER);
			if (infofile)
				GlobalFree((HGLOBAL) infofile);
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DRV_CFG_PAGE, "Drv:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		DriverInit(hkey, hdlg);
		break;

    case WM_COMMAND:
		switch (LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(),"WanpipePnp.hlp",HELP_CONTEXT,IDH_DRIVER_CFG);
			return TRUE;

		case COMBO_WANPIPE_NUMBER:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELCHANGE){
				//WanpipeNumber: ask user to confirm, cause it is a serious change.
				if(yes_no_question("Are you sure you want change the 'Wanpipe (Span) Number'?")
					== IDYES){

					Globals_New_Cfg.wanpipe_number = 
						(unsigned short)SendDlgItemMessage(hdlg, 
												COMBO_WANPIPE_NUMBER, CB_GETCURSEL, 0, 0);

					if(Globals_New_Cfg.wanpipe_number == 0){
						//Auto is selected. Restore to original AutoWanpipeNumber read from the Registry.
						Globals_New_Cfg.wanpipe_number = iAutoWanpipeNumber;
					}

					snprintf(Globals_New_Cfg.DeviceName, TMP_BUFFER_LEN, 
								"WANPIPE%d", Globals_New_Cfg.wanpipe_number);

					set_dialog_wanpipe_name(hdlg);

				}else{
					//make the current one selected
					SendDlgItemMessage(	hdlg, COMBO_WANPIPE_NUMBER, CB_SETCURSEL, 
										Globals_New_Cfg.wanpipe_number, 0);
				}
			}
			break;
		}					// process command notification
		break;
	}						// process message
	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK DrvPageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DRV_CFG_PAGE, "PSPCB_RELEASE : DrvPageCallbackProc()\n");
	}
	return TRUE;	// matters only for PSPCB_CREATE
}

////////////////////////////////////////////////////////////////////////////////

int DriverOnNotify(
			HWND    ParentHwnd,
			LPNMHDR NmHdr,
			SETUPSTUFF * Params
		    )
{
    SP_DEVINSTALL_PARAMS spDevInstall = {0};
	BOOL	configuration_check_passed = FALSE;
    
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = DriverSaveCfgData(ParentHwnd);

		Debug(DRV_CFG_PAGE, "DriverOnNotify(): PSN_APPLY");
		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE)	return HANDLE_OK;
		else									return HANDLE_ERROR;

	case PSN_QUERYCANCEL:
		Debug(DRV_CFG_PAGE, "DriverOnNotify() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
		//fall through
    default:
		Debug(DRV_CFG_PAGE, "DriverOnNotify() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
} 

////////////////////////////////////////////////////////////////////////////////

void DriverInit(HKEY hkey, HWND hdlg)
{
	char			str_wp_num[TMP_BUFFER_LEN]; 
	unsigned int	ind;
	
	///////////////////////////////////////////////////////////////////////
	//LogErrors
	///////////////////////////////////////////////////////////////////////
	if(Globals_New_Cfg.log_errors == 0){
		CheckDlgButton(hdlg, IDCHECK_IS_LOG_TO_FILE, BST_UNCHECKED);
	}else{
		CheckDlgButton(hdlg, IDCHECK_IS_LOG_TO_FILE, BST_CHECKED);
	}

	///////////////////////////////////////////////////////////////////////
	//WanpipeNumber
	///////////////////////////////////////////////////////////////////////
	SendDlgItemMessage(hdlg, COMBO_WANPIPE_NUMBER, CB_RESETCONTENT, 0, 0);
	ind = 0;
	while(ind < MAX_WP_NUMBER){
		if(ind == 0){
			snprintf(str_wp_num, TMP_BUFFER_LEN, "Auto"); 
		}else{
			snprintf(str_wp_num, TMP_BUFFER_LEN, "%d", ind); 

		}
		SendDlgItemMessage(	hdlg, COMBO_WANPIPE_NUMBER, CB_ADDSTRING, 0,
							(LPARAM)str_wp_num);
		ind++;
	}

	//make the current one selected
	SendDlgItemMessage(	hdlg, COMBO_WANPIPE_NUMBER, CB_SETCURSEL, 
						Globals_New_Cfg.wanpipe_number, 0);

	///////////////////////////////////////////////////////////////////////
	//
	///////////////////////////////////////////////////////////////////////
}

BOOL DriverSaveCfgData(HWND hdlg)
{
	char		local_tmp_buff[TMP_BUFFER_LEN]; 

	Debug(DRV_CFG_PAGE, "DriverSaveCfgData()\n");

	///////////////////////////////////////////////////////////////////
	//Log to file setting
	///////////////////////////////////////////////////////////////////
	if(IsDlgButtonChecked(hdlg, IDCHECK_IS_LOG_TO_FILE))
	{	SetRegistryStringValue(hkey, "LogErrors", "1");
		Globals_New_Cfg.log_errors = 1;
	}else
	{	SetRegistryStringValue(hkey, "LogErrors", "0");
		Globals_New_Cfg.log_errors = 0;
	}

	///////////////////////////////////////////////////////////////////
	//WanpipeNumber
	///////////////////////////////////////////////////////////////////
	Globals_New_Cfg.wanpipe_number = 
		(unsigned short)SendDlgItemMessage(hdlg, COMBO_WANPIPE_NUMBER, CB_GETCURSEL, 0, 0);
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.wanpipe_number);
	SetRegistryStringValue(hkey, "UserWanpipeNumber", local_tmp_buff);

	return TRUE;
}

void restart_device()
{
	HANDLE		hRestartWaitHandle;
	HWND		thewin;
	int			err = 0;
	// Set the PropChangeParams structure.
	SP_PROPCHANGE_PARAMS	pcp ={sizeof(SP_CLASSINSTALL_HEADER)};

	pcp.ClassInstallHeader.InstallFunction =DIF_PROPERTYCHANGE;
	pcp.Scope = DICS_FLAG_GLOBAL;

	Debug(DRV_CFG_PAGE, "********************restart_device()*********************\n");

	thewin = CreateDialog(	hInst, MAKEINTRESOURCE(DIALOG_RESTARTING),
							GetActiveWindow(), (DLGPROC)RestartingDriverDlg);
	if(thewin == NULL){
		err = 1;
		goto error;
	}

	//1-st disable our device
	pcp.StateChange = DICS_DISABLE;
	if (!SetupDiSetClassInstallParams(stuff->info, stuff->did,
			(SP_CLASSINSTALL_HEADER *)&pcp,	sizeof(pcp))){

		Debug(DRV_CFG_PAGE, "DICS_DISABLE: SetupDiSetClassInstallParams() failed!\n");
		err = 1;
		goto error;
	}

	if (!SetupDiCallClassInstaller(DIF_PROPERTYCHANGE, stuff->info, stuff->did)){
		Debug(DRV_CFG_PAGE, "DICS_DISABLE: SetupDiCallClassInstaller() failed!\n");
		err = 2;
		goto error;
	}

	//Wait for 3 seconds while device is getting disabled.
	hRestartWaitHandle = CreateEvent( NULL, FALSE, FALSE, "devchangeevent"); 
	WaitForSingleObject( hRestartWaitHandle,  3000);

	//2-nd enable our device
	pcp.StateChange = DICS_ENABLE;
	if (!SetupDiSetClassInstallParams(stuff->info, stuff->did,
			(SP_CLASSINSTALL_HEADER *)&pcp,	sizeof(pcp))){
		Debug(DRV_CFG_PAGE, "DICS_ENABLE: SetupDiSetClassInstallParams() failed!\n");
		err = 3;
		goto error;
	}

	if (!SetupDiCallClassInstaller(DIF_PROPERTYCHANGE, stuff->info, stuff->did)){
		Debug(DRV_CFG_PAGE, "DICS_DISABLE: SetupDiCallClassInstaller() failed!\n");
		err = 4;
		goto error;
	}

error:
	if(err){
		LPVOID lpMsgBuf;
		FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL );
		// Process any inserts in lpMsgBuf.
		// ...
		// Display the string.
		MessageBox( GetActiveWindow(), (LPCTSTR)lpMsgBuf, "Error",
			MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL);
		// Free the buffer.
		LocalFree( lpMsgBuf );
	}//if(err)

	DestroyWindow(thewin);
}


static LRESULT CALLBACK RestartingDriverDlg(HWND hdlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		return(1L);
	}
	return(0);
}

/*++

Routine Description:

    1.Check if the zero terminated string contains only numerical
	  characters and is not empty.
	2.If the check passed, return the integer value of the number in
	  the buffer.

Arguments:

    buff - The pointer to the string.

Return Value:

	1.	-1 if one of the checks failes.
	Or
	2.	the value of the number in the string.
	
--*/
int is_number(char * buff)
{
	unsigned int i;
	char		 *tmp = buff;

	if(!strlen(buff))
	{	Debug(DRV_CFG_PAGE, "is_number: the string is empty!!");
		return -1;
	}

	for(i = 0; i < strlen(buff); i++)
	{	if(!isdigit(*(tmp++)))
		{	return -1;
		}
	}
	Debug(DRV_CFG_PAGE, "is_number returns: %d", atoi(buff));
	return atoi(buff);
}

/////////////////////////////////////////////////////////////////////////////////////
//Wanpipe name must be the same on ALL pages
void set_dialog_wanpipe_name(HWND hdlg)
{
	char	local_tmp_buff[TMP_BUFFER_LEN]; 

	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "Device Name: %s", Globals_New_Cfg.DeviceName);

	SetDlgItemText(hdlg, IDC_SAMNAME, local_tmp_buff);
}

/////////////////////////////////////////////////////////////////////////////////////
