// Copyright (c) 1984-2006 Sangoma Technologies Inc.
//
// Sample program for reading/writing configuration
// of Sangoma Card Driver. Program reads current configuration,
// if it set to T1, then configration is changed to E1 (and vice versa)
// after that, the driver is restarted.
//
// WARNING: extra care should be taken when modifing system registry.
//
// Target: Windows 2000/XP/2003 and later. Win32 Console.
//
//
#include <windows.h>
#include <setupapi.h>	//for SetupDiXXX() functions
#include <commctrl.h>
#include <shellapi.h>
#include <devguid.h>	//for GUID_DEVCLASS_NET

#define __WINDOWS__		//must be defined before including
						//cross-platform header files.

#include <stdio.h>					//for sprintf()
#include <stdlib.h>					//for atoi()

#include <wanpipe_defines.h>		//for 'wan_udp_hdr_t'
#include <sang_status_defines.h>	//return codes
#include <sang_api.h>				//for IOCTL codes
#include <sdla_te1.h>				//for TE1 macros
#include <sdla_56k.h>				//for 56k macros
#include <sdla_remora.h>			//Analog card
#include <sdla_te3.h>				//T3 card
#include <sdla_front_end.h>			//for front-end (T1/E1/56k) commands
#include <wanpipe_cfg.h>			//for WAN_MEDIA_NONE...

/* definitions */
#define MAX_COMP_INSTID 2096
#define MAX_COMP_DESC   2096
#define MAX_FRIENDLY    2096	
#define TMP_BUFFER_LEN	500

#define T1_BIT_RATE		1536000
#define E1_BIT_RATE		1984000

#define DBG_PRNT	if(1)printf

/* globals */
TCHAR	name[MAX_FRIENDLY];

char    szCompInstanceId[MAX_COMP_INSTID];
TCHAR   szCompDescription[MAX_COMP_DESC];
TCHAR   szFriendlyName[MAX_FRIENDLY];
DWORD   dwRegType;

unsigned int TargetBitrate = 0;

/* function prototypes */
void GetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue, DWORD * pdwSize);
void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue);
void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, int iValue);
void get_set_cfg_and_disable();
void enable();

/////////////////////////////////////////////////////////////////////////////////

void GetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue, DWORD * pdwSize)
{	
	//reading twice to set dwTemp to needed value
	RegQueryValueEx(hkey, szKeyname, NULL, NULL, (unsigned char *)szValue, pdwSize);
	RegQueryValueEx(hkey, szKeyname, NULL, NULL, (unsigned char *)szValue, pdwSize);

	DBG_PRNT("GetRegistryStringValue(): %s: %s\n", szKeyname, szValue);
}

void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, LPSTR szValue)
{
	DWORD	dwSize;
	
	dwSize = strlen(szValue)+1;
    RegSetValueEx(hkey, szKeyname, 0, REG_SZ, (unsigned char *)szValue, dwSize);

	DBG_PRNT("SetRegistryStringValue(): %s: %s\n", szKeyname, szValue);
}

void SetRegistryStringValue(HKEY hkey, LPTSTR szKeyname, int iValue)
{
	DWORD	dwSize;
	char	szTemp[TMP_BUFFER_LEN];

	sprintf(szTemp, "%u", iValue);
	dwSize = strlen(szTemp)+1;
    RegSetValueEx(hkey, szKeyname, 0, REG_SZ, (unsigned char *)szTemp, dwSize);

	DBG_PRNT("SetRegistryStringValue(): %s: %s\n", szKeyname, szTemp);
}

/////////////////////////////////////////////////////////////////////////////////

/*
	1.	Go through the list of ALL "Sangoma Hardware Abstraction Driver" ports
		installed on the system.

	2.	Read current configuration for a port. If T1, change to E1 and vice versa.

	3.	Disable the port. All ports should be disabled and then re-enabled in order
		for changes to take effect. The reason for that is: no T1/E1 mixure is allowed
		on the same card.
		Before disabling a port, make sure ALL handles are closed by CloseHandle() call.
		The driver will reject STOP request if the Open Handles counter is not equal to
		zero. Refer to get_open_handles_counter() in api_common.h for finding out
		the Open Handles counter.
*/
void get_set_cfg_and_disable()
{
	int i;
	int need_reload=0; // if registry setting doesn't match to what we need
	int num=1, wanpipe_number = 1;
	SP_DEVINFO_DATA deid={sizeof(SP_DEVINFO_DATA)};
	HDEVINFO hdi = SetupDiGetClassDevs((struct _GUID *)&GUID_DEVCLASS_NET, NULL,NULL, DIGCF_PRESENT); 
	char WanpipeName[100];//DeviceName
	char Media[100];									// when comparing.
	char ActiveChannels[100];
	char Framing[10];
	char Lbo[20];
	char LineDec[10];
	char ClkMode[10];
	char E1Signalling[10];
	DWORD dwTemp;
	HKEY hkey;

	//Possible Port Names:
	//Sangoma Hardware Abstraction Driver (Port 1)
	//Sangoma Hardware Abstraction Driver (Port 2)
	//Sangoma Hardware Abstraction Driver (Port 3)
	//Sangoma Hardware Abstraction Driver (Port 4)
	//Sangoma Hardware Abstraction Driver (Port 5)
	//Sangoma Hardware Abstraction Driver (Port 6)
	//Sangoma Hardware Abstraction Driver (Port 7)
	//Sangoma Hardware Abstraction Driver (Port 8)

	//this version will search for a known port:
	//sprintf(name,"  Sangoma Hardware Abstraction Driver (Port %d)",num);

	//this version will search for all ports:
	sprintf(name,"  Sangoma Hardware Abstraction Driver");

	for (i=0;SetupDiEnumDeviceInfo(hdi,i,&deid);i++) 
	{

		BOOL fSuccess = SetupDiGetDeviceInstanceId(hdi, &deid,
			(PSTR)szCompInstanceId,MAX_COMP_INSTID, NULL);
		if (!fSuccess){
			continue;
		}

		fSuccess =SetupDiGetDeviceRegistryProperty(hdi, &deid,SPDRP_DEVICEDESC,&dwRegType,
			(BYTE*)szCompDescription,MAX_COMP_DESC,NULL);
		if (!fSuccess){
			continue;
		}

		//"FriendlyName" is optional.
		//If the same as Description, it won't be there, but try to read it any way.
		memset(szFriendlyName, 0, MAX_FRIENDLY);
		fSuccess = SetupDiGetDeviceRegistryProperty(hdi, &deid,SPDRP_FRIENDLYNAME,&dwRegType,
					 (BYTE*)szFriendlyName,MAX_FRIENDLY,NULL);

		// check if it is right Sangoma board and channel #
		if (strncmp(szCompDescription,name,strlen(name)) !=0) {	// Windows can add #2 etc - do NOT consider
			//not a Sangoma port
			continue;
		}

		if(szFriendlyName[0]){
			printf("* %s (%s)\n", szCompDescription, szFriendlyName);
		}else{
			printf("* %s\n", szCompDescription);
		}
		Sleep(1000);
#if 1

		hkey = SetupDiOpenDevRegKey(hdi, &deid, DICS_FLAG_GLOBAL, 0,
								DIREG_DRV, KEY_READ | KEY_SET_VALUE );
		if(hkey == INVALID_HANDLE_VALUE){
			printf("Error: Failed to open Registry key!!\n");
			continue;
		}

		WanpipeName[0] = '\n';
		GetRegistryStringValue(hkey, "DeviceName", WanpipeName, &dwTemp);
		printf("Wanpipe Name: %s\n", WanpipeName);

		GetRegistryStringValue(hkey, "Media",	Media, &dwTemp);
		GetRegistryStringValue(hkey, "ClkMode", ClkMode, &dwTemp);
		GetRegistryStringValue(hkey, "LBO",		Lbo, &dwTemp);
		GetRegistryStringValue(hkey, "Framing", Framing, &dwTemp);
		GetRegistryStringValue(hkey, "LDecoding", LineDec, &dwTemp);
		GetRegistryStringValue(hkey, "aft_logic_channel_0_active_ch", ActiveChannels, &dwTemp);			
		GetRegistryStringValue(hkey, "E1Signalling", E1Signalling, &dwTemp);

		switch(atoi(Media))
		{
		case WAN_MEDIA_E1:
			printf("Changing from E1 to T1...\n");
			TargetBitrate = T1_BIT_RATE;
			break;
			
		case WAN_MEDIA_T1:
			printf("Changing from T1 to E1...\n");
			TargetBitrate = E1_BIT_RATE;
			break;
		}
		Sleep(1000);

		if (TargetBitrate==E1_BIT_RATE) {
			if(atoi(Media)!=WAN_MEDIA_E1) need_reload=1;
			if(atoi(Framing)!=WAN_FR_NCRC4) need_reload=1;
			if(atoi(LineDec)!=WAN_LCODE_HDB3) need_reload=1;
			if(strcmp(ActiveChannels,"1-31")!=0) need_reload=1;
		}
		if (TargetBitrate==T1_BIT_RATE) {
			if(atoi(Media)!=WAN_MEDIA_T1) need_reload=1;
			if(atoi(Framing)!=WAN_FR_ESF) need_reload=1;
			if(atoi(LineDec)!=WAN_LCODE_B8ZS) need_reload=1;
			if(atoi(Lbo) != WAN_T1_LBO_0_DB) need_reload=1;
			if(strcmp(ActiveChannels,"1-24")!=0) need_reload=1;
		}
#endif

		if (need_reload) {
			// Set the PropChangeParams structure.
			SP_PROPCHANGE_PARAMS pcp ={sizeof(SP_CLASSINSTALL_HEADER)};
			pcp.ClassInstallHeader.InstallFunction =DIF_PROPERTYCHANGE;
			pcp.Scope = DICS_FLAG_GLOBAL;
			// disable
			pcp.StateChange = DICS_DISABLE;

			if (!SetupDiSetClassInstallParams(hdi,&deid,(SP_CLASSINSTALL_HEADER *)&pcp,	sizeof(pcp))){
				printf("Error: Line: %d\n", __LINE__);
				break;// error
			}
			if (!SetupDiCallClassInstaller(DIF_PROPERTYCHANGE,hdi,&deid)){
				printf("Error: Line: %d\n", __LINE__);
				break;// error
			}

#if 1
			// set common default values values FIXME: also set 1 group, HDLC, MTU 2048, API.
			// One group
			SetRegistryStringValue(hkey, "aft_number_of_logic_channels", 1);
			// Line mode is HDLC 
			SetRegistryStringValue(hkey, "aft_logic_channel_0_line_mode", MODE_OPTION_HDLC);
			// MTU
			SetRegistryStringValue(hkey, "aft_logic_channel_0_mtu", 2048);
			// Operational mode
			SetRegistryStringValue(hkey, "aft_logic_channel_0_operational_mode", 
										SDLA_DECODE_USEDBY_FIELD(API));

			// set Media specific default values.
			switch(TargetBitrate)
			{
			case E1_BIT_RATE:
				SetRegistryStringValue(hkey, "Media", WAN_MEDIA_E1);
				SetRegistryStringValue(hkey, "LDecoding", WAN_LCODE_HDB3);
				SetRegistryStringValue(hkey, "Framing", WAN_FR_NCRC4);
				SetRegistryStringValue(hkey, "E1Signalling", WAN_TE1_SIG_CCS);
				SetRegistryStringValue(hkey, "aft_logic_channel_0_active_ch", "1-31");			
				break;
			case T1_BIT_RATE:
				SetRegistryStringValue(hkey, "Media", WAN_MEDIA_T1);
				SetRegistryStringValue(hkey, "LDecoding", WAN_LCODE_B8ZS);
				SetRegistryStringValue(hkey, "Framing", WAN_FR_ESF);
				SetRegistryStringValue(hkey, "LBO", WAN_T1_LBO_0_DB);
				SetRegistryStringValue(hkey, "aft_logic_channel_0_active_ch", "1-24");			
				break;
			}

			if(TargetBitrate == T1_BIT_RATE || TargetBitrate == E1_BIT_RATE){

				SetRegistryStringValue(hkey, "ClkMode", WAN_NORMAL_CLK);

				SetRegistryStringValue(hkey, "ClkRefPort", 0);//IF 0 - not used, ELSE - the reference port number.

				SetRegistryStringValue(hkey, "HighImpedanceMode", WANOPT_NO);//WANOPT_NO - Off, WANOPT_YES - On.
			}
#endif
		}//if()

		//May change the Wanpipe Number too. Does not have to be T1/E1 card.
		SetRegistryStringValue(hkey, "UserWanpipeNumber", wanpipe_number);//the result is: WANPIPE1_IF0, WANPIPE2_IF0...
		wanpipe_number++;

	}//for()
	SetupDiDestroyDeviceInfoList(hdi);
}

void enable()
{
	int i;
	SP_DEVINFO_DATA deid={sizeof(SP_DEVINFO_DATA)};
	HDEVINFO hdi = SetupDiGetClassDevs((struct _GUID *)&GUID_DEVCLASS_NET, NULL,NULL, DIGCF_PRESENT); 
	// Set the PropChangeParams structure.
	SP_PROPCHANGE_PARAMS pcp ={sizeof(SP_CLASSINSTALL_HEADER)};
	pcp.ClassInstallHeader.InstallFunction =DIF_PROPERTYCHANGE;
	pcp.Scope = DICS_FLAG_GLOBAL;

	//Possible Port Names:
	//Sangoma Hardware Abstraction Driver (Port 1)
	//Sangoma Hardware Abstraction Driver (Port 2)
	//Sangoma Hardware Abstraction Driver (Port 3)
	//Sangoma Hardware Abstraction Driver (Port 4)
	//Sangoma Hardware Abstraction Driver (Port 5)
	//Sangoma Hardware Abstraction Driver (Port 6)
	//Sangoma Hardware Abstraction Driver (Port 7)
	//Sangoma Hardware Abstraction Driver (Port 8)

	//this version will search for a known port:
	//sprintf(name,"  Sangoma Hardware Abstraction Driver (Port %d)",num);

	//this version will search for all ports:
	sprintf(name,"  Sangoma Hardware Abstraction Driver");

	for (i=0;SetupDiEnumDeviceInfo(hdi,i,&deid);i++) 
	{
		char           szCompInstanceId[MAX_COMP_INSTID];
		char           szCompDescription[MAX_COMP_DESC];
		char           szFriendlyName[MAX_FRIENDLY];
		DWORD          dwRegType;

		BOOL fSuccess = SetupDiGetDeviceInstanceId(hdi, &deid,
			(PSTR)szCompInstanceId,MAX_COMP_INSTID, NULL);

		if (!fSuccess){
			continue;
		}

		fSuccess =SetupDiGetDeviceRegistryProperty(hdi, &deid,SPDRP_DEVICEDESC,
					&dwRegType,(BYTE*)szCompDescription,MAX_COMP_DESC,NULL);

		if (!fSuccess){
			continue;
		}

		memset(szFriendlyName, 0, MAX_FRIENDLY);
		SetupDiGetDeviceRegistryProperty(hdi, &deid,SPDRP_FRIENDLYNAME,&dwRegType,
					 (BYTE*)szFriendlyName,MAX_FRIENDLY,NULL);
		// check if it is right Sangoma board and channel #
		if (strncmp(szCompDescription,name,strlen(name)) !=0) {	// Windows can add #2 etc - do NOT consider
			//not a Sangoma port
			continue;
		}

		if(szFriendlyName[0]){
			printf("@ %s (%s)\n", szCompDescription, szFriendlyName);
		}else{
			printf("@ %s\n", szCompDescription);
		}
		Sleep(1000);

		// enable
		pcp.StateChange = DICS_ENABLE;
		if (!SetupDiSetClassInstallParams(hdi,&deid,(SP_CLASSINSTALL_HEADER *)&pcp,	sizeof(pcp))){
			printf("Error: Line: %d\n", __LINE__);
			break;// error
		}
		if (!SetupDiCallClassInstaller(DIF_PROPERTYCHANGE,hdi,&deid)){
			printf("Error: Line: %d\n", __LINE__);
			break;// error
		}

	}//if()
	SetupDiDestroyDeviceInfoList(hdi);
}

int main()
{
	printf("\nStopping Ports...\n");
	get_set_cfg_and_disable();

	printf("\nStarting Ports...\n");
	enable();
	return 0;
}
