/***********************************************************************************
	api_common.h

	Abstracted Sangoma API calls.
************************************************************************************/

#ifndef _API_COMMON_H_
#define _API_COMMON_H_

//////////////////////////////////////////////////////////////////
int DoManagementCommand(
	HANDLE drv, 
	wan_udp_hdr_t* packet
	);

#define DO_COMMAND(wan_udp)	DoManagementCommand(hGeneralCommands, &wan_udp);

struct fun_protocol {
	int		protocol_id;
	unsigned char   prot_name[10];
	/*
	config_t*	config;
	usage_t*	usage;
	main_t*		main;
	dis_trace_t*	disable_trace;
	main_menu_t*	get_main_menu;
	cmd_menu_t*	get_cmd_menu;
	ft1_cfg_t*	read_FT1_status;
	ft1_cfg_t*	set_FT1_mode;
	*/
	unsigned char	mbox_offset;
};

struct fun_protocol function_lookup[] = {
	{ WANCONFIG_FR,			"fr",		0 }, 
	{ WANCONFIG_MFR,		"fr",		0 },
	{ WANCONFIG_PPP,		"ppp",		0 }, 
	{ WANCONFIG_CHDLC,		"chdlc",	0 },//supported - S514 
	{ WANCONFIG_ATM,		"atm",		2 }, 
	{ WANCONFIG_AFT,		"aft",		0 },//supported
	{ WANCONFIG_AFT_TE1,	"aft",		0 },//supported
	{ WANCONFIG_AFT_ANALOG,	"aft",		0 },//supported
	{ WANCONFIG_ADSL,		"adsl",		2 }, 
	{ WANCONFIG_SS7,		"ss7",		2 },
	{ WANCONFIG_X25, 		"x25",		0 }, 
	{ WANCONFIG_BITSTRM,	"bitstrm",	0 },
	{ WANCONFIG_BSC, 		"bsc",		0 }, 
	{ WANCONFIG_HDLC, 		"hdlc",		0 }, 
	{ 0,					"N/A",		0 }, 
};

#define DECODE_PROT(prot)((prot==WANCONFIG_FR)?"Frame Relay":	\
			  (prot==WANCONFIG_MFR)?"Frame Relay":				\
	          (prot==WANCONFIG_PPP)?"PPP Point-to-Point":		\
	          (prot==WANCONFIG_CHDLC)?"Cisco HDLC":				\
	          (prot==WANCONFIG_X25)?"X25":						\
	          (prot==WANCONFIG_ADSL)?"ADSL: Eth,IP,PPP/ATM":	\
			  (prot==WANCONFIG_ATM)?"ATM":						\
			  (prot==WANCONFIG_AFT)?"AFT T1/E1":				\
			  (prot==WANCONFIG_AFT_TE1)?"AFT T1/E1":			\
			  (prot==WANCONFIG_AFT_ANALOG)?"AFT Analog":		\
							"Unknown")

static int GetWANConfig( void );
static unsigned char get_wan_udphdr_data_byte(unsigned char off);
static unsigned char *get_wan_udphdr_data_ptr(unsigned char off);
static unsigned char set_wan_udphdr_data_byte(unsigned char off, unsigned char data);
static int get_fe_type(unsigned char* adapter_type);
static void read_te1_56k_stat(void);
static void set_lb_modes(unsigned char type, unsigned char mode);
static void operational_stats (void);

//////////////////////////////////////////////////////////////////

static unsigned char get_wan_udphdr_data_byte(unsigned char off) 
{
	int i=0; 

	for (;;){

		if (function_lookup[i].protocol_id == 0){
			printf("\nProtocol: '%s' support not compiled in!\n\n", 
					DECODE_PROT(wan_protocol)); 
			break;
		};
	
		if (function_lookup[i].protocol_id == wan_protocol){

			return (((unsigned char*)&wan_udp.wan_udphdr_data[0])+
				function_lookup[i].mbox_offset)[off];
		}

		i++;
	}

	return 0;
}

static int GetWANConfig( void )
{
	int err = WAN_TRUE;

	/* Get Protocol type */ 
	wan_udp.wan_udphdr_command	= WAN_GET_PROTOCOL;
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if (wan_udp.wan_udphdr_return_code){
		printf("Error: Command WAN_GET_PROTOCOL failed! return code: 0x%X",
			wan_udp.wan_udphdr_return_code);
		return WAN_FALSE;
	}

	wan_protocol = wan_udp.wan_udphdr_data[0];

	printf("Device %s running protocol: %s\n",
		device_name, DECODE_PROT(wan_protocol));
	 
	return err;
} 

static int get_fe_type(unsigned char* adapter_type)
{
	int err = WAN_TRUE;

	/* Read Adapter Type */
	wan_udp.wan_udphdr_command = WAN_GET_MEDIA_TYPE;
	wan_udp.wan_udphdr_data[0] = WAN_MEDIA_NONE;
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: Command WAN_GET_MEDIA_TYPE failed! return code: 0x%X",
			wan_udp.wan_udphdr_return_code);
		return WAN_FALSE;
	}

	*adapter_type =	get_wan_udphdr_data_byte(0);

	printf("Front End Type: ");
	switch(*adapter_type)
	{
	case WAN_MEDIA_NONE:
		printf("Serial");
		break;

	case WAN_MEDIA_T1:
		printf("T1");
		break;

	case WAN_MEDIA_E1:
		printf("E1");
		break;

	case WAN_MEDIA_56K:
		printf("56K");
		break;

	case WAN_MEDIA_FXOFXS:
		printf("Aanalog");
		break;

	default:
		printf("Unknown");
		err = WAN_FALSE;
	}
	printf("\n");

	return err;
}

static unsigned char *get_wan_udphdr_data_ptr(unsigned char off) 
{
	int i=0; 
	for (;;){
		if (function_lookup[i].protocol_id == 0){
			printf("\nProtocol: '%s' support not compiled in!\n\n", 
					DECODE_PROT(wan_protocol)); 
			break;
		};
	
		if (function_lookup[i].protocol_id == wan_protocol){

			return &(((unsigned char*)&wan_udp.wan_udphdr_data[0])+
				function_lookup[i].mbox_offset)[off];
		}
		i++;
	}

	return 0;
}

static unsigned char set_wan_udphdr_data_byte(unsigned char off, unsigned char data) 
{
	int i=0; 
	for (;;){
		if (function_lookup[i].protocol_id == 0){
			printf("\nProtocol: '%s' support not compiled in!\n\n", 
					DECODE_PROT(wan_protocol)); 
			break;
		};
	
		if (function_lookup[i].protocol_id == wan_protocol){

			(((unsigned char*)&wan_udp.wan_udphdr_data[0])+
				function_lookup[i].mbox_offset)[off] = data;

			return 0;
		}
		i++;
	}

	return -1;
}

static void read_te1_56k_stat(void)
{
	unsigned char* data = NULL;
	unsigned long 	alarms = 0x00;
	sdla_te_pmon_t*	pmon = NULL;

	switch(adapter_type)
	{
	case WAN_MEDIA_T1:
	case WAN_MEDIA_E1:
	case WAN_MEDIA_56K:
		;//do nothing
		break;

	default:
		printf("Command invalid for Adapter Type %d.\n", adapter_type);
		return;
	}

	/* Read T1/E1/56K alarms and T1/E1 performance monitoring counters */
	wan_udp.wan_udphdr_command = WAN_FE_GET_STAT;
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if (wan_udp.wan_udphdr_return_code != 0){
		printf("Failed to read T1/E1/56K statistics.\n");
		return;
	}

	data = get_wan_udphdr_data_ptr(0);

#if 0
	if (wan_protocol == WANCONFIG_CHDLC){
		data = &wan_udp.wan_udphdr_chdlc_data[0];
	}else{
		data = &wan_udp.wan_udphdr_data[0];
	}
#endif

	alarms = *(unsigned long*)data;
	if (adapter_type == WAN_MEDIA_T1 || adapter_type == WAN_MEDIA_E1){
		printf("***** %s: %s Alarms *****\n\n",
			device_name, (adapter_type == WAN_MEDIA_T1) ? "T1" : "E1");
		printf("ALOS:\t%s\t| LOS:\t%s\n", 
				WAN_TE_ALOS_ALARM(alarms), 
				WAN_TE_LOS_ALARM(alarms));
		printf("RED:\t%s\t| AIS:\t%s\n", 
				WAN_TE_RED_ALARM(alarms), 
				WAN_TE_AIS_ALARM(alarms));
		if (adapter_type == WAN_MEDIA_T1){ 
			printf("YEL:\t%s\t| OOF:\t%s\n", 
					WAN_TE_YEL_ALARM(alarms), 
					WAN_TE_OOF_ALARM(alarms));
		}else{
			printf("OOF:\t%s\n", 
					WAN_TE_OOF_ALARM(alarms));
		}
	}else{
		printf("***** %s: 56K CSU/DSU Alarms *****\n\n\n", device_name);
	 	printf("In Service:\t\t%s\tData mode idle:\t\t%s\n",
			 	INS_ALARM_56K(alarms), 
			 	DMI_ALARM_56K(alarms));
	
	 	printf("Zero supp. code:\t%s\tCtrl mode idle:\t\t%s\n",
			 	ZCS_ALARM_56K(alarms), 
			 	CMI_ALARM_56K(alarms));

	 	printf("Out of service code:\t%s\tOut of frame code:\t%s\n",
			 	OOS_ALARM_56K(alarms), 
			 	OOF_ALARM_56K(alarms));
		
	 	printf("Valid DSU NL loopback:\t%s\tUnsigned mux code:\t%s\n",
			 	DLP_ALARM_56K(alarms), 
			 	UMC_ALARM_56K(alarms));

	 	printf("Rx loss of signal:\t%s\t\n",
			 	RLOS_ALARM_56K(alarms)); 
	}

	if (adapter_type != WAN_MEDIA_T1 && adapter_type != WAN_MEDIA_E1){
		return;
	}
	pmon = (sdla_te_pmon_t*)&data[sizeof(unsigned long)];
/*
//FIXME:
	printf("\n\n***** %s: %s Performance Monitoring Counters *****\n\n",
			device_name, (adapter_type == WAN_MEDIA_T1) ? "T1" : "E1");
	printf("Framing Bit Error:\t%ld\tLine Code Violation:\t%ld\n", 
			pmon->frm_bit_error,
			pmon->lcv);
	if (adapter_type == WAN_MEDIA_T1){
		printf("Out of Frame Errors:\t%ld\tBit Errors:\t\t%ld\n", 
				pmon->oof_errors,
				pmon->bit_errors);
	}else{
		printf("Far End Block Errors:\t%ld\tCRC Errors:\t%ld\n", 
				pmon->far_end_blk_errors,
				pmon->crc_errors);
	}
*/
	return;
}

static void set_lb_modes(unsigned char type, unsigned char mode)
{
	switch(adapter_type)
	{
	case WAN_MEDIA_T1:
	case WAN_MEDIA_E1:
	case WAN_MEDIA_56K:
		;//do nothing
		break;

	default:
		printf("Command invalid for Adapter Type %d.\n", adapter_type);
		return;
	}
	
	wan_udp.wan_udphdr_command	= WAN_FE_SET_LB_MODE;
	wan_udp.wan_udphdr_data_len	= 2;

	set_wan_udphdr_data_byte(0,type);
	set_wan_udphdr_data_byte(1,mode);

#if 0
	if (wan_protocol == WANCONFIG_CHDLC){
		wan_udp.wan_udphdr_chdlc_data[0] = type;
		wan_udp.wan_udphdr_chdlc_data[1] = mode;
	}else{
		wan_udp.wan_udphdr_data[0] = type;
		wan_udp.wan_udphdr_data[1] = mode;
	}
#endif
	DO_COMMAND(wan_udp);
	if (wan_udp.wan_udphdr_return_code != 0){
		printf("Failed to %s line loopback mode. rc: 0x%X\n",
			((mode == WAN_TE1_ACTIVATE_LB) ? "activate" : "deactivate"), 
			wan_udp.wan_udphdr_return_code);
	}else{
		printf("Line loopback mode is %s!\n",
			(mode == WAN_TE1_ACTIVATE_LB) ? "activated" : "deactivated");
	}
	return;
}


static void operational_stats (void)
{
	net_device_stats_t *stats;
	wan_udp.wan_udphdr_command = READ_OPERATIONAL_STATS; 
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command READ_OPERATIONAL_STATS failed!\n");
		return;
	}

	stats = (net_device_stats_t *)get_wan_udphdr_data_ptr(0);

	printf("**** OPERATIONAL_STATS ****\n");

	printf("\ttx_dropped\t: %lu\n", stats->tx_dropped);
	printf("\ttx_packets\t: %lu\n", stats->tx_packets);
	printf("\ttx_bytes\t: %lu\n", stats->tx_bytes);
	printf("\n");

	//receive stats
	printf("\trx_packets\t: %lu\n", stats->rx_packets);
	printf("\trx_bytes\t: %lu\n", stats->rx_bytes);

	printf("\trx_errors\t: %lu\n", stats->rx_errors);
	printf("\trx_frame_errors\t: %lu\n", stats->rx_frame_errors);

	printf("\trx_packets_discarded_rx_q_full\t: %lu\n",
		stats->rx_packets_discarded_rx_q_full);
	printf("\trx_packets_discarded_excessive_length\t: %lu\n",
		stats->rx_packets_discarded_excessive_length);
	
	printf("\trx_packets_discarded_too_short\t: %lu\n", 
		stats->rx_packets_discarded_too_short);
	printf("\n");

	//hardware level stats
	//AFT card
	printf("\trx_fifo_errors\t: %lu\n", stats->rx_fifo_errors);
	printf("\ttx_fifo_errors\t: %lu\n", stats->tx_fifo_errors);
	//S508, S514
	printf("\tnumber_of_times_rx_buffers_out_of_synch_with_card\t: %lu\n",
		stats->number_of_times_rx_buffers_out_of_synch_with_card);

	//BitStream mode only - counter of data blocks transmitted automatically,
	//because no User Data was available. Transmit underrun counter from user
	//point of view.
	printf("\ttx_idle_data\t: %lu\n", stats->tx_idle_data);
}

static void flush_operational_stats (void)
{
	wan_udp.wan_udphdr_command = FLUSH_OPERATIONAL_STATS; 
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command FLUSH_OPERATIONAL_STATS failed!\n");
		return;
	}

	printf("Command FLUSH_OPERATIONAL_STATS was successful.\n");
}

static void read_api_driver_version (void)
{
	PDRIVER_VERSION version;
	wan_udp.wan_udphdr_command = READ_CODE_VERSION; 
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command READ_CODE_VERSION failed!\n");
		return;
	}

	version = (PDRIVER_VERSION)get_wan_udphdr_data_ptr(0);

	printf("\nAPI version\t: %d,%d,%d,%d\n",
		version->major, version->minor, version->minor1, version->minor2);
	printf("\n");

	printf("Command READ_CODE_VERSION was successful.\n");
}

//get RBS being received
static char get_rbs(rbs_management_t *rbs_management_ptr)
{
	wan_udp.wan_udphdr_command = WAN_GET_RBS_BITS;
	memcpy(&wan_udp.wan_udphdr_data[0], rbs_management_ptr, sizeof(rbs_management_t));
	wan_udp.wan_udphdr_data_len = sizeof(rbs_management_t);

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command WAN_GET_RBS_BITS failed!\n");
		return 1;
	}

	rbs_management_ptr = (rbs_management_t *)&wan_udp.wan_udphdr_data[0];

	printf("**** WAN_GET_RBS_BITS OK ****\n");

	//printf("ABCD_bits (HEX): 0x%02X\n", rbs_management_ptr->ABCD_bits);

	printf("Channel: %d, RX RBS: A:%1d B:%1d C:%1d D:%1d\n",
			rbs_management_ptr->channel, 
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_A) ? 1 : 0,
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_B) ? 1 : 0,
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_C) ? 1 : 0,
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_D) ? 1 : 0);
	return 0;
}

//set RBS to be transmitted
static char set_rbs(rbs_management_t *rbs_management_ptr)
{
	wan_udp.wan_udphdr_command = WAN_SET_RBS_BITS;
	memcpy(&wan_udp.wan_udphdr_data[0], rbs_management_ptr, sizeof(rbs_management_t));
	wan_udp.wan_udphdr_data_len = sizeof(rbs_management_t);

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command WAN_SET_RBS_BITS failed!\n");
		return 1;
	}

	printf("**** WAN_SET_RBS_BITS OK ****\n");

	//printf("ABCD_bits (HEX): 0x%02X\n", rbs_management_ptr->ABCD_bits);

	printf("Channel: %d, TX RBS: A:%1d B:%1d C:%1d D:%1d\n",
			rbs_management_ptr->channel, 
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_A) ? 1 : 0,
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_B) ? 1 : 0,
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_C) ? 1 : 0,
			(rbs_management_ptr->ABCD_bits & WAN_RBS_SIG_D) ? 1 : 0);
	return 0;
}

static int get_interface_configuration(if_cfg_t *wanif_conf_ptr)
{
	wan_udp.wan_udphdr_command = READ_CONFIGURATION;
	wan_udp.wan_udphdr_data_len = sizeof(if_cfg_t);

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command READ_CONFIGURATION failed!\n");
		return 1;
	}
	memcpy(wanif_conf_ptr, get_wan_udphdr_data_ptr(0), sizeof(if_cfg_t));

	printf("**** READ_CONFIGURATION ****\n");
	printf("Operational Mode\t: %s\n", wanif_conf_ptr->usedby);
	printf("Active Channels \t: 0x%08X\n", wanif_conf_ptr->active_ch);
	printf("Interface Number\t: %u\n", wanif_conf_ptr->interface_number);
	printf("Media type\t\t: %u\n", wanif_conf_ptr->media);
	if(wanif_conf_ptr->media == WAN_MEDIA_FXOFXS){
		switch(wanif_conf_ptr->sub_media)
		{
		case MOD_TYPE_FXS:
			printf("Media sub-type\t\t: FXS\n");
			break;
		case MOD_TYPE_FXO:
			printf("Media sub-type\t\t: FXO\n");
			break;
		default:
			printf("Media sub-type\t\t: Unknown!!\n");
			break;
		}
	}
	printf("****************************\n");

	return 0;
}

static int get_open_handles_counter()
{
	wan_udp.wan_udphdr_command = GET_OPEN_HANDLES_COUNTER; 
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		printf("Error: command GET_OPEN_HANDLES_COUNTER failed!\n");
		return -1;
	}
	//printf("open handles counter: %d\n", *(int*)&wan_udp.wan_udphdr_data[0]);
	return *(int*)&wan_udp.wan_udphdr_data[0];
}

///////////////////////////////////////////////////////////////////////////

static int tdmv_api_ioctl(wanpipe_tdm_api_cmd_t *api_cmd, int cmd)
{
	DWORD			ln;
    unsigned char	id = 0;
	int				err = 0;

	wan_udp.wan_udphdr_request_reply = 0x01;
	wan_udp.wan_udphdr_id = id;
   	wan_udp.wan_udphdr_return_code = WAN_UDP_TIMEOUT_CMD;

	wan_udp.wan_udphdr_command	= WAN_TDMV_API_IOCTL;
	wan_udp.wan_udphdr_data_len	= sizeof(wanpipe_tdm_api_cmd_t);

	api_cmd->cmd = cmd;
	//copy data from caller's buffer to driver's buffer
	memcpy(	wan_udp.wan_udphdr_data, 
			(void*)api_cmd,
			sizeof(wanpipe_tdm_api_cmd_t));

	if(DeviceIoControl(
			hGeneralCommands,
			IoctlManagementCommand,
			(LPVOID)&wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPVOID)&wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
			) == FALSE){
		err = 1;
	}else{
		err = 0;
	}

	if(err){
		printf("Error: tdmv_api_ioctl(): DeviceIoControl failed!!\n");
		return 1;
	}
	if(wan_udp.wan_udphdr_return_code != WAN_CMD_OK){
		printf("Error: tdmv_api_ioctl(): command failed! Return code: 0x%X.\n",
			wan_udp.wan_udphdr_return_code);
		return 2;
	}

	//copy data from driver's buffer to caller's buffer
	memcpy(	(void*)api_cmd,
			wan_udp.wan_udphdr_data, 
			sizeof(wanpipe_tdm_api_cmd_t));
	return 0;
}

static int wanpipe_api_ioctl(wan_cmd_api_t *api_cmd)
{
	DWORD			ln;
    unsigned char	id = 0;
	int				err = 0;

	wan_udp.wan_udphdr_request_reply = 0x01;
	wan_udp.wan_udphdr_id = id;
   	wan_udp.wan_udphdr_return_code = WAN_UDP_TIMEOUT_CMD;

	wan_udp.wan_udphdr_command	= SIOC_WANPIPE_API;
	wan_udp.wan_udphdr_data_len	= sizeof(wan_cmd_api_t);

	//copy data from caller's buffer to driver's buffer
	memcpy(	wan_udp.wan_udphdr_data, 
			(void*)api_cmd,
			sizeof(wan_cmd_api_t));

	if(DeviceIoControl(
			hGeneralCommands,
			IoctlManagementCommand,
			(LPVOID)&wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPVOID)&wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
			) == FALSE){
		err = 1;
	}else{
		err = 0;
	}

	if(err){
		printf("Error: wanpipe_api_ioctl(): DeviceIoControl failed!!\n");
		return 1;
	}
	if(wan_udp.wan_udphdr_return_code != WAN_CMD_OK){
		printf("Error: wanpipe_api_ioctl(): command failed! Return code: 0x%X.\n",
			wan_udp.wan_udphdr_return_code);
		return 2;
	}

	//copy data from driver's buffer to caller's buffer
	memcpy(	(void*)api_cmd,
			wan_udp.wan_udphdr_data, 
			sizeof(wan_cmd_api_t));
	return 0;
}

#endif//API_COMMON
