// Copyright (c) 1984-2007 Sangoma Technologies Inc.
//
// Sample program for the following Sangoma APIs:
//	1. HDLC  Streaming API
//	2. Bit Streaming API
//	3. Frame Relay API
//	4. CHDLC API
//	5. LAPD (Q.921) API
//	6. "TDM VOICE API"
//
//Note 1: the header files are in "include" directory,
//		your compiler should know where to look for
//		the Header files.
//
//
// Target: Windows XP/2003 and later. Win32 Console.
//
// For Version beta-6,0,2,2 and later of the API driver (sdladrv.sys)
//
#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>
#include <stddef.h>		//for offsetof()

//////////////////////////////////////////////////////////////////

#define __WINDOWS__	//must be defined for Windows compiler

#include <wanpipe_defines.h>	//for 'wan_udp_hdr_t'
#include <sang_status_defines.h>//return codes
#include <sang_api.h>			//for IOCTL codes
#include <sdla_te1.h>			//TE1 macros
#include <sdla_56k.h>			//56k macros
#include <sdla_remora.h>		//Analog card
#include <sdla_te3.h>			//T3 card
#include <sdla_front_end.h>		//front-end (T1/E1/56k) commands
#include <wanpipe_cfg.h>		//for RBS commands, Media Types (WAN_MEDIA_NONE...)

#include <wanpipe_tdm_api.h>	//for TDMV API
#include <sdla_aft_te1.h>		//for Wanpipe API

//////////////////////////////////////////////////////////////////
//Device Names formed as follows:
// "\\\\.\\WANPIPE1_IF0" - corresponds to Group 1 on WANPIPE1
//
// "\\\\.\\WANPIPE1_IF1" - corresponds to Group 2 on WANPIPE1
//
// "\\\\.\\WANPIPE1_IF2" - corresponds to Group 3 on WANPIPE1
//
// "\\\\.\\WANPIPE2_IF0" - corresponds to Group 1 on WANPIPE2
//
// "\\\\.\\WANPIPE2_IF1" - corresponds to Group 2 on WANPIPE2
//
//and so on...
//
//(WANPIPE1 is Port 1 on the first installed card)

#define DEV_NAME_LEN	100
char device_name[DEV_NAME_LEN];

/*
The Shared Event allows to use WaitForMultipleObjects() for receiving
data and events on MULTIPLE Device Handles using SINGLE "Rx thread".
*/
//#define SHARED_EVENT

//////////////////////////////////////////////////////////////////
HANDLE	hGeneralCommands = INVALID_HANDLE_VALUE;
HANDLE	hRx = INVALID_HANDLE_VALUE;
HANDLE	hTx = INVALID_HANDLE_VALUE;

if_cfg_t wanif_conf_struct;

//////////////////////////////////////////////////////////////////
//receive stuff
RX_DATA_STRUCT rx_data;
ULONG rx_frames_count = 0;
ULONG rx_bytes_count = 0;
//for counting frames with CRC/Abort errors
ULONG bad_rx_frames_count = 0;

//////////////////////////////////////////////////////////////////
//transmit stuff
TX_DATA_STRUCT tx_data;
HANDLE hTxWaitHandle = NULL;
HANDLE hTxCompleteWaitHandle = NULL;

ULONG tx_bytes_count = 0;
ULONG tx_frames_count = 0;
UCHAR tx_test_byte = 0;

//mutex for synchronizing access to the TX data
HANDLE tx_data_mutex;
#define TX_MUTEX_TIMEOUT	1000//1 second

//NOTE: if using BitStream mode, TX_LENGTH must be EXACTLY the same
//		as MTU for the Device.
#define TX_LENGTH	50

//////////////////////////////////////////////////////////////////
//IOCTL management structures and variables
wan_udp_hdr_t	wan_udp;
int				protocol_cb_size = sizeof(wan_mgmt_t)+sizeof(wan_cmd_t)+1;
int 			wan_protocol = 0;
unsigned char	adapter_type = 0;

#include "api_common.h"

//////////////////////////////////////////////////////////////////

//Optional mutex for synchronizing access to 'stdout' between the threads,
//just to make the output to look neat.
HANDLE print_mutex = NULL;
#define PRINT_MUTEX_TIMEOUT	1000// 1 second

static wan_cmd_api_t wanpipe_api_cmd;
static api_tx_hdr_t *tx_hdr = (api_tx_hdr_t *)wanpipe_api_cmd.data;

static wanpipe_tdm_api_cmd_t tdm_api_cmd;

//////////////////////////////////////////////////////////////////

UCHAR 
DoWriteCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	);

USHORT 
DoReadCommand(
	HANDLE drv, 
	RX_DATA_STRUCT * pRx
	);

DWORD 
RxThreadFunc(
	LPDWORD lpdwParam
	);

static int read_data();

DWORD 
TxThreadFunc(
	LPDWORD lpdwParam
	);

void 
PrintRxData(
	RX_DATA_STRUCT* pRx
	);

void 
PrintTdmApiEvent(
	RX_DATA_STRUCT* pRx
	);

int 
main_loop(
	DWORD *user_selection
	);

void cleanup();

int DoManagementCommand(
	HANDLE drv, 
	wan_udp_hdr_t* packet
	);

HANDLE
open_api_device(
	char *api_device_name
	);

UCHAR 
DoSetIdleTxBufferCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	);

UCHAR 
DoSetSharedEventCommand(
	HANDLE drv, 
	PREGISTER_EVENT event
	);

static int get_user_number();

int main(int argc, char* argv[])
{
	DWORD	user_selection = 0;
    DWORD	dwThreadId; 
	HANDLE	current_process = NULL;
	HANDLE	hRxThread = NULL;
	HANDLE  hTxThread = NULL;

	////////////////////////////////////////////////////////////////////////////
	//initialize Device Name with the default name
	_snprintf(device_name , DEV_NAME_LEN, "\\\\.\\%s", "WANPIPE1_IF0");

	////////////////////////////////////////////////////////////////////////////
	//check, if user provided Device Name as a command line argument
	if(argc == 2){
		_snprintf(device_name , DEV_NAME_LEN, "\\\\.\\%s", argv[1]);
	}
	printf("Using Device Name: %s\n", device_name);

	////////////////////////////////////////////////////////////////////////////
	//Open device for general commands
	hGeneralCommands = open_api_device(device_name);
    if (hGeneralCommands == INVALID_HANDLE_VALUE){
		printf("Unable to open %s for general commands!\n", device_name);
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//initialize PrintToScreen mutex
	print_mutex = CreateMutex(NULL, FALSE, NULL);
	if(print_mutex == NULL){
		printf("Failed to create print_mutex!!!!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//get current protocol
	if(GetWANConfig() == WAN_FALSE){
		printf("Failed to get current protocol!\n");
		cleanup();
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//get Front End Type (T1/E1/Analog...)
	if (get_fe_type(&adapter_type) == WAN_FALSE){
		printf("Failed to get Front End Type!\n");
		cleanup();
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//may need interface configuration, so get it now
	if(get_interface_configuration(&wanif_conf_struct)){
		printf("Failed to get Interface Configuration!\n");
		cleanup();
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	tx_data_mutex = CreateMutex(NULL, FALSE, NULL);
	if(tx_data_mutex == NULL){
		printf("Failed to create 'tx_data_mutex'!!!!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	current_process = GetCurrentProcess();
	//this program 'talks' direcly to hardware, it has to be real time class.
	if(SetPriorityClass(current_process, REALTIME_PRIORITY_CLASS) == 0){
		printf("Failed to set program priority!\n!");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//Start a thread for receiving data only
    hRxThread = CreateThread( 
        NULL,                       /* no security attributes        */ 
        0,                          /* use default stack size        */ 
        (LPTHREAD_START_ROUTINE)RxThreadFunc, /* thread function     */ 
        &user_selection,            /* argument to thread function   */ 
        0,                          /* use default creation flags    */ 
        &dwThreadId					/* returns the thread identifier */ 
		);

	if(hRxThread == NULL){
		printf("Failed to create Rx thread!!\n");
		cleanup();
		return 1;
	}

	if(SetThreadPriority(hRxThread, THREAD_PRIORITY_TIME_CRITICAL) == FALSE){
		printf("Failed to set Rx thread priority!!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//create Tx wait object
	hTxWaitHandle = CreateEvent( NULL, FALSE, FALSE, NULL);
	if(hTxWaitHandle == NULL){
		printf("Failed to create Tx event!!\n");
		cleanup();
		return 1;
	}

	//Start a thread for writing only
	hTxThread = CreateThread( 
		NULL,                       /* no security attributes        */ 
		0,                          /* use default stack size        */ 
		(LPTHREAD_START_ROUTINE) TxThreadFunc, /* thread function    */ 
		&user_selection,            /* argument to thread function   */ 
		0,                          /* use default creation flags    */ 
		&dwThreadId					/* returns the thread identifier */ 
		);

	if(hTxThread == NULL){
		printf("Failed to create Tx thread!!\n");
		cleanup();
		return 1;
	}

	if(SetThreadPriority(hTxThread, THREAD_PRIORITY_TIME_CRITICAL) == FALSE){
		printf("failed to set Tx thread priority!!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//create Tx Complete wait object
	hTxCompleteWaitHandle = CreateEvent( NULL, FALSE, FALSE, NULL);
	if(hTxCompleteWaitHandle == NULL){
		printf("Failed to create Tx Complete event!!\n");
		cleanup();
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//enter main loop
	main_loop(&user_selection);

	////////////////////////////////////////////////////////////////////////////
	//don't forget the cleanup.
	cleanup();

	WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);
	printf("\nSample application exiting.(press any key)\n");
	ReleaseMutex(print_mutex);

	getch();
	return 0;
}


int main_loop(DWORD *user_selection)
{
	unsigned char	idle_tx_test_byte = 0;
	DWORD			mresult;

	////////////////////////////////////////////////////////////////////////////
	// Loop around waiting for user to request the transmission of data
	// or to do some other command.
	do{
		WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);

		printf("\n Press 'q' to quit,\n");
		printf("\t or 't' to transmit data.\n");
		printf("\t or 's' to get device statistics.\n");
		printf("\t or 'f' to flush (reset) device statistics.\n");
		printf("\t or 'v' to read API driver version.\n");
		if(adapter_type == WAN_MEDIA_T1 || adapter_type == WAN_MEDIA_E1){
			printf("\t or 'a' to get T1/E1 alarms.\n");
			//RBS (CAS) commands
			printf("\t or 'g' to get RBS bits.\n");
			printf("\t or 'r' to set RBS bits.\n");
		}
		printf("\t or 'i' to set Tx idle data buffer (BitStream only).\n");
		switch(adapter_type)
		{
		case WAN_MEDIA_T1:
			//those commands valid only for T1
			printf("\t or 'l' to send 'activate remote loop back' signal.\n");
			printf("\t or 'd' to send 'deactivate remote loop back' signal.\n");
			break;

		case WAN_MEDIA_FXOFXS:
			switch(wanif_conf_struct.sub_media)
			{
			case MOD_TYPE_FXS:
				printf("\t or 'e' to listen to test tones on a phone connected to the A200-FXS\n");
				printf("\t or 'c' to ring/stop ring phone connected to the A200-FXS\n");
				printf("\t or 'n' to enable/disable reception of ON/OFF Hook events on A200-FXS\n");
				printf("\t or 'm' to enable DTMF events (on SLIC chip) on A200-FXS\n");
				printf("\t or 'j' to enable/disable reception of Ring Trip events on A200-FXS\n");
				break;

			case MOD_TYPE_FXO:
				printf("\t or 'u' to enable/disable reception of Ring Detect events on A200-FXO\n");
				printf("\t or 'h' to transmit ON/OFF hook signals on A200-FXO\n");
				break;
			}
			break;
		}
		printf("\t or 'o' to enable DTMF events (on Octasic chip)\n");
		ReleaseMutex(print_mutex);

		*user_selection = tolower(getch());
		printf("\n");

		switch(*user_selection)
		{
		case 't':
			//mutex is used to protect the global "tx_data" buffer
			mresult = WaitForSingleObject(tx_data_mutex, TX_MUTEX_TIMEOUT);
			if (mresult != WAIT_OBJECT_0) {
				printf("Failed to get tx mutex!!!\n");		
				break;
			}

			//initialize tx_data structure
			tx_data.api_header.data_length = TX_LENGTH;
			tx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;
			//set the actual data for transmission
			memset(tx_data.data, tx_test_byte, TX_LENGTH);
	
			ReleaseMutex(tx_data_mutex);

			//Data buffer is ready for transmission. Tell TX thread to run:
			SetEvent(hTxWaitHandle);
			break;

		case 's':
			//get device statistics
			operational_stats();
			break;

		case 'f':
			//flush (reset) device statistics
			flush_operational_stats();
			break;

		case 'v':
			//read API driver version
			read_api_driver_version();
			break;

		case 'a':
			//read T1/E1/56k alarms
			read_te1_56k_stat();
			break;

		case 'l'://send activate remote loop back signal
			set_lb_modes(WAN_TE1_TX_LB_MODE, WAN_TE1_ACTIVATE_LB);
			break;

		case 'd'://send deactivate remote loop back signal
			set_lb_modes(WAN_TE1_TX_LB_MODE, WAN_TE1_DEACTIVATE_LB);
			break;

		case 'g'://read RBS bits
			{
				rbs_management_t rbs_management_struct;

				printf("Type Channel number and press <Enter>:\n");
				rbs_management_struct.channel = get_user_number();
				//IMPORTANT: this call must be thread safe - use mutex or critical section around it.
				get_rbs(&rbs_management_struct);
			}
			break;

		case 'r'://set RBS bits
			{
				static rbs_management_t rbs_management_struct = {0,0};

				printf("Type Channel number and press <Enter>:\n");
				rbs_management_struct.channel = get_user_number();//channels (Time Slots). Valid values: 1 to 24.

				/*	bitmap - set as needed: WAN_RBS_SIG_A |	WAN_RBS_SIG_B | WAN_RBS_SIG_C | WAN_RBS_SIG_D;

					In this example make bits A and B to change each time,
					so it's easy to see the change on the receiving side.
				*/
				if(rbs_management_struct.ABCD_bits == WAN_RBS_SIG_A){
					rbs_management_struct.ABCD_bits = WAN_RBS_SIG_B;
				}else{
					rbs_management_struct.ABCD_bits = WAN_RBS_SIG_A;
				}
				//IMPORTANT: this call must be thread safe - use mutex or critical section around it.
				set_rbs(&rbs_management_struct);
			}
			break;

		case 'i':
			//set data in Idle Tx buffer in the driver
			tx_data.api_header.data_length = TX_LENGTH;
			tx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;
			//set the actual data for transmission
			memset(tx_data.data, idle_tx_test_byte, TX_LENGTH);

			if(	DoSetIdleTxBufferCommand(hGeneralCommands, &tx_data)){
				//error
				printf("DoSetIdleTxBufferCommand() failed!! Check messages log.\n");
				break;
			}

			if(tx_data.api_header.operation_status != SANG_STATUS_SUCCESS){
				//error
				printf("DoSetIdleTxBufferCommand() failed!!.\n");
				break;
			}
			idle_tx_test_byte++;
			break;

		case 'c':
			printf("Press 'e' to START ring, 'd' to STOP ring.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tx_hdr->wp_api_tx_hdr_event_mode = WP_API_EVENT_ENABLE;//start
				break;
			case 'd':
			default:
				tx_hdr->wp_api_tx_hdr_event_mode = WP_API_EVENT_DISABLE;//stop

			}
			tx_hdr->wp_api_tx_hdr_event_type = WP_API_EVENT_RING;
			wanpipe_api_ioctl(&wanpipe_api_cmd);
			break;

		case 'e':
			printf("Press 'e' to START a Tone, 'd' to STOP a Tone.\n");
			printf("\n");

			tx_hdr->wp_api_tx_hdr_event_type = WP_API_EVENT_TONE;

			switch(tolower(getch()))
			{
			case 'e':
				printf("Press 'r' for Ring Tone, 'd' for Dial Tone, 'b' for Busy Tone, 'c' for Congestion Tone.\n");
				printf("\n");
				switch(tolower(getch()))
				{
				case 'r':
					tx_hdr->wp_api_tx_hdr_event_tone = WP_API_EVENT_TONE_RING;
					break;
				case 'd':
					tx_hdr->wp_api_tx_hdr_event_tone = WP_API_EVENT_TONE_DIAL;
					break;
				case 'b':
					tx_hdr->wp_api_tx_hdr_event_tone = WP_API_EVENT_TONE_BUSY;
					break;
				case 'c':
				default:
					tx_hdr->wp_api_tx_hdr_event_tone = WP_API_EVENT_TONE_CONGESTION;
					break;
				}
				tx_hdr->wp_api_tx_hdr_event_mode = WP_API_EVENT_ENABLE;//start
				break;

			case 'd':
			default:
				tx_hdr->wp_api_tx_hdr_event_mode = WP_API_EVENT_DISABLE;//stop

			}

			wanpipe_api_ioctl(&wanpipe_api_cmd);
			break;

		case 'n':
			printf("Press 'e' to ENABLE Rx Hook Events, 'd' to DISABLE Rx Hook Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_ENABLE_RXHOOK_EVENTS);
				break;
			case 'd':
			default:
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_DISABLE_RXHOOK_EVENTS);
			}
			break;

		case 'm':
			//Enable/Disable DTMF events on SLIC chip.
			//On Analog (A200) card only.
			printf("Press 'e' to ENABLE Remora DTMF Events, 'd' to DISABLE Remora DTMF Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_ENABLE_RM_DTMF_EVENTS);
				break;
			case 'd':
			default:
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_DISABLE_RM_DTMF_EVENTS);
			}
			break;

		case 'o':
			//Enable DTMF events on Octasic chip. 
			//For both Analog (A200) and T1/E1 (A104D) cards, but only if the chip is present.
			printf("Press 'e' to ENABLE Octasic DTMF Events, 'd' to DISABLE Octasic DTMF Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_ENABLE_DTMF_EVENTS);
				break;
			case 'd':
			default:
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_DISABLE_DTMF_EVENTS);
			}
			break;

		case 'u':
			//Enable/Disable Ring Detect events on FXO. 
			printf("Press 'e' to ENABLE Rx Ring Detect Events, 'd' to DISABLE Rx Ring Detect Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_ENABLE_RING_DETECT_EVENTS);
				break;
			case 'd':
			default:
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_DISABLE_RING_DETECT_EVENTS);
			}
			break;

		case 'j':
			//Enable/Disable Ring Trip events on FXS. 
			printf("Press 'e' to ENABLE Rx Ring Trip Events, 'd' to DISABLE Rx Ring Trip Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_ENABLE_RING_TRIP_DETECT_EVENTS);
				break;
			case 'd':
			default:
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_DISABLE_RING_TRIP_DETECT_EVENTS);
			}
			break;

		case 'h':
			printf("Press 'e' to transmit OFF hook signal, 'd' to transmit ON hook signal.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_EVENT_TXSIG_OFFHOOK);
				break;
			case 'd':
			default:
				tdmv_api_ioctl(&tdm_api_cmd, SIOC_WP_TDM_EVENT_TXSIG_ONHOOK);
			}
			break;

		default:
			if(*user_selection != 'q'){
				printf("\nUnknown command.\n");
			}
			break;
		}//switch(*user_selection)

	}while(*user_selection != 'q');

	return 0;
}

static int get_user_number()
{
	int result = 1;
	int retry_counter = 0;

	while(scanf("%d", &result) == 0){
		fflush( stdin );
		printf("\nError: Not a numerical input!! Must be from 1 to 24.\n");
		if(retry_counter++ > 10){
			printf("giving up...\n");
			result = 1;
			break;
		}
	}//while()

	printf("User input: %d\n", result);
	return result;
}

HANDLE open_api_device(char *api_device_name)
{
	return CreateFile(	api_device_name, 
						GENERIC_READ | GENERIC_WRITE, 
						FILE_SHARE_READ | FILE_SHARE_WRITE,
						(LPSECURITY_ATTRIBUTES)NULL, 
						OPEN_EXISTING,
						FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH,
						(HANDLE)NULL
						);
}

//It is very important to close ALL open
//handles, because the API will continue
//receive data until the LAST handel is closed.
void cleanup()
{
	if(print_mutex != NULL){
		WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);
	}

	printf("cleanup()\n");

	if(hGeneralCommands != INVALID_HANDLE_VALUE){
		int hndls_counter = get_open_handles_counter();
		if(hndls_counter > 0){
			printf("There are %d API Handles to be closed.\n", hndls_counter);
		}
		printf("Closing hGeneralCommands.\n");
		CloseHandle(hGeneralCommands);
		hGeneralCommands = INVALID_HANDLE_VALUE;
	}

	if(hRx != INVALID_HANDLE_VALUE){
		printf("Closing hRx.\n");
		CloseHandle(hRx);
		hRx = INVALID_HANDLE_VALUE;
	}

	if(hTx != INVALID_HANDLE_VALUE){
		printf("Closing hTx.\n");
		CloseHandle(hTx);
		hTx = INVALID_HANDLE_VALUE;
	}

	if(hTxWaitHandle != NULL){
		CloseHandle(hTxWaitHandle);
		hTxWaitHandle = NULL;
	}

	if(hTxCompleteWaitHandle != NULL){
		CloseHandle(hTxCompleteWaitHandle);
		hTxCompleteWaitHandle = NULL;
	}

	if(print_mutex != NULL){
		ReleaseMutex(print_mutex);
	}
}

// Blocking write command.
UCHAR 
DoWriteCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	)
{
	DWORD ln;

	if(DeviceIoControl(
			drv,
			IoctlWriteCommand,
			(LPVOID)pTx,
			(ULONG)sizeof(TX_DATA_STRUCT),
			(LPVOID)pTx,
			sizeof(TX_DATA_STRUCT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
		//check messages log
		return 1;
	}else{
		return 0;
	}
}

// Blocking read command.
USHORT 
DoReadCommand(
	HANDLE drv, 
	RX_DATA_STRUCT * pRx
	)
{
	DWORD ln;

	if (DeviceIoControl(
			drv,
			IoctlReadCommand,
			(LPVOID)NULL,
			0L,
			(LPVOID)pRx,
			sizeof(RX_DATA_STRUCT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
		//check messages log
		return 1;
	}else{
		return 0;
	}
}

// set data in Idle Transmit buffer (BitStream only).
UCHAR 
DoSetIdleTxBufferCommand(
	HANDLE drv, 
	TX_DATA_STRUCT * pTx
	)
{
	DWORD ln;

	if(DeviceIoControl(
			drv,
			IoctlSetIdleTxBuffer,
			(LPVOID)pTx,
			(ULONG)sizeof(TX_DATA_STRUCT),
			(LPVOID)pTx,
			sizeof(TX_DATA_STRUCT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
		//check messages log
		return 1;
	}else{
		return 0;
	}
}


UCHAR 
DoSetSharedEventCommand(
	HANDLE drv, 
	PREGISTER_EVENT event
	)
{
	DWORD ln;

	if(DeviceIoControl(
			drv,
			IoctlSetSharedEvent,
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
		//check messages log
		return 1;
	}else{
		return 0;
	}
}

int DoManagementCommand(
	HANDLE drv, 
	wan_udp_hdr_t* wan_udp
	)
{
	DWORD ln;

    static unsigned char id = 0;
	int data_len = protocol_cb_size+wan_udp->wan_udphdr_data_len;

	wan_udp->wan_udphdr_request_reply = 0x01;
	wan_udp->wan_udphdr_id = id;
   	wan_udp->wan_udphdr_return_code = WAN_UDP_TIMEOUT_CMD;

	if(DeviceIoControl(
			drv,
			IoctlManagementCommand,
			(LPVOID)wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPVOID)wan_udp,
			sizeof(wan_udp_hdr_t),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
		//check messages log
		return 1;
	}else{
		return 0;
	}
}


// Read Thread
DWORD 
RxThreadFunc(
	LPDWORD lpdwParam
	)
{ 
#if defined(SHARED_EVENT)
	REGISTER_EVENT	event;
	DWORD			mresult;
#endif

    printf("\nRxThreadFunc:  thread parameter=%d\n", *lpdwParam); 

	//open another handle strictly for reading
	hRx = open_api_device(device_name);
    if(hRx == INVALID_HANDLE_VALUE){
		printf("\nUnable to open %s for reading!\n", device_name);
		return 1;
	}

#if defined(SHARED_EVENT)
    event.hEvent = CreateEvent( NULL, FALSE, FALSE, NULL);
	if(!event.hEvent ) {
         printf("CreateEvent error = %d !!\n", GetLastError() );
		 return 2;
	}

	//Provide API driver the "Shared Event" object structure.
	if(DoSetSharedEventCommand(hRx, &event)){
		//system error
		printf("DoSetSharedEventCommand() failed!! Check messages log.\n");
		 return 3;
	}

	if(event.operation_status != SANG_STATUS_SUCCESS){
		//user error
		printf("DoSetSharedEventCommand() failed!! err: %d(%s).\n",
			event.operation_status, SDLA_DECODE_SANG_STATUS(event.operation_status));
		 return 4;
	}
#endif

	while( *lpdwParam != 'q'){	

#if defined(SHARED_EVENT)
		//wait for "rx wait object" to be signalled
		mresult = WaitForMultipleObjects(1, &event.hEvent, FALSE, INFINITE);
		if(mresult != WAIT_OBJECT_0){
			printf("WaitForSingleObject() for \"Shared Event\" failed!!\n");
			break;
		}

		do{
			if(read_data()){
				break;
			}
/*
			//It is possible, there is more than one Data Frame or Event in
			//rx queue, when Shared Event was set to signalled state.
			printf("max_rx_queue_length: %d current_number_of_frames_in_rx_queue: %d\n",  
				rx_data.api_header.rx_h.max_rx_queue_length,
				rx_data.api_header.rx_h.current_number_of_frames_in_rx_queue);
*/
		}while(rx_data.api_header.rx_h.current_number_of_frames_in_rx_queue > 0);
#else
		if(read_data()){
			break;
		}
#endif

	}//while()

	WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);
	printf("Read Thread is terminating...\n");
	ReleaseMutex(print_mutex);

    return 0; 
}

static int read_data()
{
	api_header_t	*pri;
		
	if(DoReadCommand(hRx, &rx_data) ){
		//error
		printf("DoReadCommand() failed!! Check messages log.\n");
		return 1;
	}

	//use our special buffer at rxdata to hold received data
	pri = &rx_data.api_header;

	switch(pri->operation_status)
	{
	case SANG_STATUS_RX_DATA_TIMEOUT:
#if defined(SHARED_EVENT)
		//when using Shared Event, this should never happen
		printf("Error: Timeout on read!!\n");
		return 2;
#else
		//No data in READ_CMD_TIMEOUT, try again. (NOT and error)
		printf("Timeout on read.\n");
#endif
		break;

	case SANG_STATUS_BUFFER_TOO_SMALL:
		//Recieved data longer than the pre-configured maximum.
		//Maximum length is set in 'Interface Properties',
		//in the 'Device Manager'.
		printf("Received data longer than buffer passed to API.\n");
		break;

	case SANG_STATUS_LINE_CONNECTED:
		printf("Rx:new line state: SANG_STATUS_LINE_CONNECTED\n");
		break;

	case SANG_STATUS_LINE_DISCONNECTED:
		printf("Rx:new line state: SANG_STATUS_LINE_DISCONNECTED\n");
		break;

	case SANG_STATUS_RX_DATA_AVAILABLE:
		WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);
		PrintRxData(&rx_data);
		ReleaseMutex(print_mutex);
		break;

	case SANG_STATUS_TDM_EVENT_AVAILABLE:
		WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);
		PrintTdmApiEvent(&rx_data);
		ReleaseMutex(print_mutex);
		break;

	default:
		printf("Rx:Unknown Operation Status: %d\n", pri->operation_status);
		break;
	}//switch()
		
	return 0;
}
 
void PrintRxData(RX_DATA_STRUCT* pRx)
{
	api_header_t*	pri;
	USHORT			datlen, ln;
	PUCHAR			data;

	pri = &pRx->api_header;

	//
	//NOTE: if running in BitStream mode, there will be TOO MUCH to print 
	datlen = pri->data_length;
	data =  pRx->data;

	printf("Rx datlen : %d. Data :\n", datlen);

	for(ln = 0; ln < datlen; ln++){
		//do not print more than 100 bytes
		if(ln == 99){
			printf("...\n");
			break;
		}
		if(ln % 20 == 0){
			printf("\n");
		}
		printf("%02X ", data[ln]);
	}
	printf("\n");

	rx_frames_count++;
	rx_bytes_count += datlen;
}

void PrintTdmApiEvent(RX_DATA_STRUCT* pRx)
{
	wp_tdm_api_rx_hdr_t	*tdm_api_rx_hdr;

	tdm_api_rx_hdr = (wp_tdm_api_rx_hdr_t*)pRx->data;

	switch(tdm_api_rx_hdr->wp_tdm_api_event_type)
	{
	case WP_TDM_EVENT_DTMF:
		printf("DTMF Event: Channel: %d, Digit: %c (Port: %s, Type:%s)!\n",
			tdm_api_rx_hdr->wp_tdm_api_event_channel,
			tdm_api_rx_hdr->wp_tdm_api_event_dtmf_digit,
			(tdm_api_rx_hdr->wp_tdm_api_event_dtmf_port == WAN_EC_CHANNEL_PORT_ROUT)?"ROUT":"SOUT",
			(tdm_api_rx_hdr->wp_tdm_api_event_dtmf_type == WAN_EC_TONE_PRESENT)?"PRESENT":"STOP");		
		break;

	case WP_TDM_EVENT_RXHOOK:
		printf("RXHOOK Event: %s! (0x%X)\n", 
			WAN_EVENT_RXHOOK_DECODE(tdm_api_rx_hdr->wp_tdm_api_event_rxhook_state),
			tdm_api_rx_hdr->wp_tdm_api_event_rxhook_state);
		break;

	case WP_TDM_EVENT_RING_DETECT:
		printf("RING Event: %s! (0x%X)\n",	
			WAN_EVENT_RING_DECODE(tdm_api_rx_hdr->wp_tdm_api_event_ring_state),
			tdm_api_rx_hdr->wp_tdm_api_event_ring_state);
		break;

	case WP_TDM_EVENT_RING_TRIP:
		printf("RING TRIP Event: %s! (0x%X)\n", 
			WAN_EVENT_RING_TRIP_DECODE(tdm_api_rx_hdr->wp_tdm_api_event_ring_state),
			tdm_api_rx_hdr->wp_tdm_api_event_ring_state);
		break;

	default:
		printf("Unknown TDM API Event: %d\n", tdm_api_rx_hdr->wp_tdm_api_event_type);
		break;
	}
}


// Write Thread
DWORD 
TxThreadFunc(
	LPDWORD lpdwParam
	)
{ 
	DWORD			dwWaitResult;
	DWORD			mresult;
	BOOLEAN			stop_tx = FALSE;
	
    printf("\nTxThreadFunc:  thread parameter=%d\n", *lpdwParam); 
	
	//open another handle strictly for writing
	hTx = open_api_device(device_name);
    if (hTx == INVALID_HANDLE_VALUE){
		printf("\nUnable to open %s for writing!\n", device_name);
		return 1;
	}
	
	//start the transmission
	while(*lpdwParam != 'q' && stop_tx == FALSE){

		//Wait for tx event to be set to signaled state.
		dwWaitResult = WaitForSingleObject(hTxWaitHandle, 1000);
		
		switch(dwWaitResult)
		{
		case WAIT_TIMEOUT:
			continue;
		case WAIT_ABANDONED:
			printf("Tx WAIT_ABANDONED\n.");
			stop_tx = TRUE;
			break;
		}

		if(stop_tx == TRUE){
			break;
		}

		//mutex is used to protect the global "tx_data" buffer
		mresult = WaitForSingleObject(tx_data_mutex, TX_MUTEX_TIMEOUT);
		if (mresult != WAIT_OBJECT_0) {
			printf("TxThreadFunc(): Failed to get tx mutex!!!\n");		
			break;
		}

		//queue data for transmission
		if(	DoWriteCommand(hTx, &tx_data)){
			//error
			printf("DoWriteCommand() failed!! Check messages log.\n");
			break;
		}
		
		//check that frame was transmitted
		switch(tx_data.api_header.operation_status)
		{
		case SANG_STATUS_SUCCESS:

			tx_bytes_count += tx_data.api_header.data_length;
			tx_frames_count++;
			tx_test_byte++;

			WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);

			printf("Frame queued for transmission.\n");

			//Some useful information about API's internal transmit queue
			//is available after each successful transmission:
			printf("max_tx_queue_length: %d current_number_of_frames_in_tx_queue: %d\n",  
				tx_data.api_header.tx_h.max_tx_queue_length,
				tx_data.api_header.tx_h.current_number_of_frames_in_tx_queue);
		
			ReleaseMutex(print_mutex);

			//signal to anyone who may wait for transmission completion
			SetEvent(hTxCompleteWaitHandle);
			break;
					
		case SANG_STATUS_TX_TIMEOUT:
			//error
			//It means that tx buffers are constanly full,
			//usually because the line is disconnected
			printf("****** SANG_STATUS_TX_TIMEOUT ******\n");
			//Check messages log or look at statistics.
			Sleep(1000);
			break;
					
		case SANG_STATUS_TX_DATA_TOO_LONG:
			//Attempt to transmit data longer than the pre-configured maximum.
			//Maximum length is set in 'Interface Properties',
			//in the 'Device Manager'.
			printf("****** SANG_STATUS_TX_DATA_TOO_LONG ******\n");
			Sleep(1000);
			break;
					
		case SANG_STATUS_TX_DATA_TOO_SHORT:
			//Minimum is 1 byte  for Primary   port,
			//			 2 bytes for Secondary port
			printf("****** SANG_STATUS_TX_DATA_TOO_SHORT ******\n");
			Sleep(1000);
			break;

		case SANG_STATUS_LINE_DISCONNECTED:
			//Front end monitoring is enabled and Line is
			//in disconnected state.
			//Check the T1/E1 line is in "Connected" state,
			//alse check the Alarms and the message log.
			printf("****** SANG_STATUS_LINE_DISCONNECTED ******\n");
			Sleep(1000);
			break;

		default:
			printf("Unknown return code (0x%X) on transmission!\n",
				tx_data.api_header.operation_status);
			Sleep(1000);
			break;
		}//switch()
			
		ReleaseMutex(tx_data_mutex);
		
	}//while()

	WaitForSingleObject(print_mutex, PRINT_MUTEX_TIMEOUT);
	printf("Write Thread is terminating...\n");
	ReleaseMutex(print_mutex);

    return 0; 
}

