// 
// Copyright (c) 1984-2007 Sangoma Technologies Inc.
//
// sample.cpp: 
//	Sample C++ program for the following Sangoma APIs:
//	1. HDLC  Streaming API
//	2. Bit Streaming API
//	3. Frame Relay API
//	4. CHDLC API
//	5. LAPD (Q.921) API
//	6. "TDM VOICE API"
//
//Note 1: the header files are in "include" directory,
//		your compiler should know where to look for
//		the Header files.
//
//
// Target: Windows XP/2003 and later. Win32 Console.
//
// For Version beta-6,0,2,2 and later of the API driver (sdladrv.sys)
//

//////////////////////////////////////////////////////////////////
//Device Names formed as follows:
// "\\\\.\\WANPIPE1_IF0" - corresponds to Group 1 on WANPIPE1
//
// "\\\\.\\WANPIPE1_IF1" - corresponds to Group 2 on WANPIPE1
//
// "\\\\.\\WANPIPE1_IF2" - corresponds to Group 3 on WANPIPE1
//
// "\\\\.\\WANPIPE2_IF0" - corresponds to Group 1 on WANPIPE2
//
// "\\\\.\\WANPIPE2_IF1" - corresponds to Group 2 on WANPIPE2
//
//and so on...
//
//(WANPIPE1 is Port 1 on the first installed card)
//////////////////////////////////////////////////////////////////

#include "sangoma_interface.h"

//globals:
//mutex for synchronizing access to 'stdout' between the threads
HANDLE	print_mutex = NULL;
HANDLE	tdm_event_mutex = NULL;

callback_functions_t callback_functions;

//function prototypes:
static int got_rx_data(void *sang_if_ptr, void *rx_data);
static void got_TdmApiEvent(void *sang_if_ptr, void *event_data);
//////////////////////////////////////////////////////////////////

typedef struct{
	void			*sang_if_ptr;
	RX_DATA_STRUCT	event_data;
}TDM_API_EVENT_THREAD_PARAM;

DWORD TdmApiEventThreadFunc(LPDWORD lpdwParam);
//////////////////////////////////////////////////////////////////
#define DBG_MAIN	if(1)printf
#define ERR_MAIN	if(1)printf
#define INFO_MAIN	if(1)printf
//////////////////////////////////////////////////////////////////

sangoma_interface* init(int wanpipe_number, int interface_number)
{
	sangoma_interface	*sang_if = NULL;

	DBG_MAIN("init()\n");

	callback_functions.got_rx_data = got_rx_data;
	callback_functions.got_TdmApiEvent = got_TdmApiEvent;

	sang_if = new sangoma_interface(wanpipe_number, interface_number);
	if(sang_if->init(&callback_functions)){
		delete sang_if;
		return NULL;
	}

	DBG_MAIN("init(): OK\n");
	return sang_if;
}

void cleanup(sangoma_interface	*sang_if)
{
	DBG_MAIN("cleanup()\n");

	if(sang_if){
		delete sang_if;
	}
}

int start(sangoma_interface	*sang_if)
{
	DBG_MAIN("start()\n");
	return sang_if->run();
}

void stop(sangoma_interface	*sang_if)
{
	DBG_MAIN("stop()\n");
	sang_if->stop();
}

void PrintRxData(RX_DATA_STRUCT* pRx)
{
	api_header_t*	pri;
	USHORT			datlen;
	PUCHAR			data;
	static	rx_counter = 0;

	pri = &pRx->api_header;

	//
	//NOTE: if running in BitStream mode, there will be TOO MUCH to print 
	datlen = pri->data_length;
	data =  pRx->data;

	rx_counter++;
	//if((rx_counter % 20) == 0){
		printf("Rx counter:%d, Rx datlen : %d. Data :\n", rx_counter, datlen);
	//}

#if 0
	for(int ln = 0; ln < datlen; ln++){
		//do not print more than 100 bytes
		if(ln == 99){
			printf("...\n");
			break;
		}
		if(ln % 20 == 0){
			printf("\n");
		}
		printf("%02X ", data[ln]);
	}
	printf("\n");
#endif
}

static int got_rx_data(void *sang_if_ptr, void *rx_data)
{
	sangoma_interface	*sang_if;
	//Do something with data received from Sangoma interface.
	//
	//Fore example, transimit back everything what was received:
	sang_if = (sangoma_interface*)sang_if_ptr;

	//sang_if->transmit((TX_RX_DATA_STRUCT*)rx_data);

	PrintRxData((RX_DATA_STRUCT*)rx_data);

	return 0;
}

static void got_TdmApiEvent(void *sang_if_ptr, void *event_data)
{
	TDM_API_EVENT_THREAD_PARAM	*param;
	DWORD						dwThreadId;

	WaitForSingleObject(tdm_event_mutex, 1000);

	param = (TDM_API_EVENT_THREAD_PARAM*)malloc(sizeof(TDM_API_EVENT_THREAD_PARAM));
	if(param == NULL){
		ERR_MAIN("Failed to allocate memory for 'Event Thread parameter'!!\n");
		ReleaseMutex(tdm_event_mutex);
		return;
	}

	memcpy(&param->event_data, event_data, sizeof(RX_DATA_STRUCT));
	param->sang_if_ptr = sang_if_ptr;

	//
	//Handling of Events must be done OUTSIDE of the REAL-TIME Rx thread.
	//
	//Create a special thread for Event hadling.
	//
	if(CreateThread( 
        NULL,                       /* no security attributes        */ 
        0,                          /* use default stack size        */ 
        (LPTHREAD_START_ROUTINE)TdmApiEventThreadFunc, /* thread function     */ 
        param,						/* argument to thread function   */ 
        0,                          /* use default creation flags    */ 
        &dwThreadId					/* returns the thread identifier */ 
		) == NULL){
		ERR_MAIN("Failed to create 'TdmApiEvent' thread!!\n");
	}
	ReleaseMutex(tdm_event_mutex);
}

DWORD TdmApiEventThreadFunc(LPDWORD lpdwParam)
{ 
	TDM_API_EVENT_THREAD_PARAM	*param;
	RX_DATA_STRUCT				*pEvent;
	sangoma_interface			*sang_if;
	wp_tdm_api_rx_hdr_t			*tdm_api_rx_hdr;

	WaitForSingleObject(tdm_event_mutex, 1000);

	param = (TDM_API_EVENT_THREAD_PARAM*)lpdwParam;

	pEvent = &param->event_data;
	sang_if = (sangoma_interface*)param->sang_if_ptr;

	tdm_api_rx_hdr = (wp_tdm_api_rx_hdr_t*)pEvent->data;


	DBG_MAIN( "TdmApiEventThreadFunc:: ifname: %s\n", sang_if->device_name);

	switch(tdm_api_rx_hdr->wp_tdm_api_event_type)
	{
	case WP_TDM_EVENT_DTMF:
		DBG_MAIN("DTMF Event: Channel: %d, Digit: %c (Port: %s, Type:%s)!\n",
			tdm_api_rx_hdr->wp_tdm_api_event_channel,
			tdm_api_rx_hdr->wp_tdm_api_event_dtmf_digit,
			(tdm_api_rx_hdr->wp_tdm_api_event_dtmf_port == WAN_EC_CHANNEL_PORT_ROUT)?"ROUT":"SOUT",
			(tdm_api_rx_hdr->wp_tdm_api_event_dtmf_type == WAN_EC_TONE_PRESENT)?"PRESENT":"STOP");		

		break;

	case WP_TDM_EVENT_RXHOOK:
		DBG_MAIN("RXHOOK Event: %s! (0x%X)\n", 
			WAN_EVENT_RXHOOK_DECODE(tdm_api_rx_hdr->wp_tdm_api_event_rxhook_state),
			tdm_api_rx_hdr->wp_tdm_api_event_rxhook_state);

		sang_if->last_hook_state_event = tdm_api_rx_hdr->wp_tdm_api_event_rxhook_state;

		break;

	case WP_TDM_EVENT_RING_DETECT:
		DBG_MAIN("RING Event: %s! (0x%X)\n",	
			WAN_EVENT_RING_DECODE(tdm_api_rx_hdr->wp_tdm_api_event_ring_state),
			tdm_api_rx_hdr->wp_tdm_api_event_ring_state);
		break;

	case WP_TDM_EVENT_RING_TRIP:
		DBG_MAIN("RING TRIP Event: %s! (0x%X)\n", 
			WAN_EVENT_RING_TRIP_DECODE(tdm_api_rx_hdr->wp_tdm_api_event_ring_state),
			tdm_api_rx_hdr->wp_tdm_api_event_ring_state);
		break;

	default:
		ERR_MAIN("Unknown TDM API Event: %d\n", tdm_api_rx_hdr->wp_tdm_api_event_type);
		break;
	}

	free(lpdwParam);
	ReleaseMutex(tdm_event_mutex);

	//Done with the Event, exit the thread.
	return 0;
}


int main(int argc, char* argv[])
{
	char				rc;
	HANDLE				current_process = NULL;
	char				user_selection;
	unsigned int		wanpipe_number = 1;	
	unsigned int		interface_number = 0;	
	sangoma_interface	*sang_if = NULL;
	TX_DATA_STRUCT		local_tx_data;
	UCHAR				tx_test_byte = 0;
	int					data_length = 50;

	////////////////////////////////////////////////////////////////////////////
	//initialize mutex objects
	print_mutex = CreateMutex(NULL, FALSE, NULL);
	if(print_mutex == NULL){
		printf("Failed to create print_mutex!!!!\n");
		return 1;
	}

	tdm_event_mutex = CreateMutex(NULL, FALSE, NULL);
	if(tdm_event_mutex == NULL){
		printf("Failed to create tdm_event_mutex!!!!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	current_process = GetCurrentProcess();
	//this program 'talks' direcly to hardware, it has to be real time class.
	if(SetPriorityClass(current_process, REALTIME_PRIORITY_CLASS) == 0){
		ERR_MAIN("Failed to set program priority!\n!");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//User may provide Wanpipe Number and Interface Number as a command line arguments:
	if(argc == 3){
		wanpipe_number = atoi(argv[1]);	
		interface_number = atoi(argv[2]);
	}
	printf("Using wanpipe_number: %d, interface_number: %d\n", 
		wanpipe_number, interface_number);

	sang_if = init(wanpipe_number, interface_number);

	if(sang_if == NULL){
		return 1;
	}

	rc = start(sang_if);
	if(rc){
		cleanup(sang_if);
		return rc;
	}

	do{
		INFO_MAIN("Press 'q' to quit the program.\n");
		INFO_MAIN("Press 't' to transmit data.\n");
		INFO_MAIN("Press 's' to get Operational Statistics.\n");
		INFO_MAIN("Press 'f' to reset (flush) Operational Statistics.\n");
		user_selection = tolower(getch());
		switch(user_selection)
		{
		case 'q':
			break;
		case 't':
			int cnt;
			for(cnt = 0; cnt < 1; cnt++){
				local_tx_data.api_header.data_length = data_length;
				local_tx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;

				//set the actual data for transmission
				memset(local_tx_data.data, tx_test_byte, data_length);

				sang_if->transmit(&local_tx_data);

				tx_test_byte++;
			}
			break;

		case 's':
			sang_if->operational_stats();
			break;

		case 'f':
			sang_if->flush_operational_stats();
			break;

		default:
			INFO_MAIN("Invalid command.\n");
		}

	}while(user_selection != 'q');

	stop(sang_if);
	cleanup(sang_if);

	return 0;
}

