
/***************************************************************************
                          sangoma_interface.h  -  description
                             -------------------
    begin                : Mon Dec 4 2006
    copyright            : (C) 2006 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SANGOMA_INTERFACE_H
#define SANGOMA_INTERFACE_H

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>
#include <stddef.h>		//for offsetof()


#define __WINDOWS__
#include <wanpipe_defines.h>	//for 'wan_udp_hdr_t'
#include <sang_status_defines.h>//return codes
#include <sang_api.h>			//for IOCTL codes
#include <sdla_remora.h>		//Analog card
#include <wanpipe_cfg.h>		//for RBS commands, Media Types (WAN_MEDIA_NONE...)

#include <wanpipe_tdm_api.h>	//for TDMV API
#include <sdla_aft_te1.h>		//for Wanpipe API

#include "sample_cpp.h"


/**
  *@author David Rokhvarg
  */

struct func_protocol {
	int		protocol_id;
	unsigned char   prot_name[10];
	/*
	config_t*	config;
	usage_t*	usage;
	main_t*		main;
	dis_trace_t*	disable_trace;
	main_menu_t*	get_main_menu;
	cmd_menu_t*	get_cmd_menu;
	ft1_cfg_t*	read_FT1_status;
	ft1_cfg_t*	set_FT1_mode;
	*/
	unsigned char	mbox_offset;
};


static struct func_protocol function_lookup[] = {
	{ WANCONFIG_FR,			"fr",		0 }, 
	{ WANCONFIG_MFR,		"fr",		0 },
	{ WANCONFIG_PPP,		"ppp",		0 }, 
	{ WANCONFIG_CHDLC,		"chdlc",	2 }, 
	{ WANCONFIG_ATM,		"atm",		2 }, 
	{ WANCONFIG_AFT,		"aft",		0 },//supported
	{ WANCONFIG_AFT_TE1,	"aft",		0 },//supported
	{ WANCONFIG_AFT_ANALOG,	"aft",		0 },//supported
	{ WANCONFIG_ADSL,		"adsl",		2 }, 
	{ WANCONFIG_SS7,		"ss7",		2 },
	{ WANCONFIG_X25, 		"x25",		0 }, 
	{ WANCONFIG_BITSTRM,	"bitstrm",	0 },
	{ WANCONFIG_BSC, 		"bsc",		0 }, 
	{ WANCONFIG_HDLC, 		"hdlc",		0 }, 
	{ 0,					"N/A",		0 }, 
};

#define DECODE_PROT(prot)((prot==WANCONFIG_FR)?"Frame Relay":	\
			  (prot==WANCONFIG_MFR)?"Frame Relay":				\
	          (prot==WANCONFIG_PPP)?"PPP Point-to-Point":		\
	          (prot==WANCONFIG_CHDLC)?"Cisco HDLC":				\
	          (prot==WANCONFIG_X25)?"X25":						\
	          (prot==WANCONFIG_ADSL)?"ADSL: Eth,IP,PPP/ATM":	\
			  (prot==WANCONFIG_ATM)?"ATM":						\
			  (prot==WANCONFIG_AFT)?"AFT T1/E1":				\
			  (prot==WANCONFIG_AFT_TE1)?"AFT T1/E1":			\
			  (prot==WANCONFIG_AFT_ANALOG)?"AFT Analog":		\
							"Unknown")

class sangoma_interface {

	//////////////////////////////////////////////////////////////////
	HANDLE		hGeneralCommands;
	HANDLE		hRx;
	HANDLE		hTx;

	//////////////////////////////////////////////////////////////////
	//receive stuff
	ULONG		rx_frames_count;
	ULONG		rx_bytes_count;
	//for counting frames with CRC/Abort errors
	ULONG		bad_rx_frames_count;
	HANDLE		hRxThread;
	DWORD		dwRxThreadId; 

	//////////////////////////////////////////////////////////////////
	//transmit stuff
	HANDLE		hTxWaitHandle;
	ULONG		tx_bytes_count;
	ULONG		tx_frames_count;
	UCHAR		tx_test_byte;

	//////////////////////////////////////////////////////////////////
	//IOCTL management structures and variables
	wan_udp_hdr_t	wan_udp;
	int				protocol_cb_size;
	int 			wan_protocol;
	unsigned char	adapter_type;

	wanpipe_tdm_api_cmd_t tdm_api_cmd;

	void PrintRxData(RX_DATA_STRUCT *pRx);
	void PrintTdmApiEvent(RX_DATA_STRUCT *pRx);
	void cleanup();

	wan_cmd_api_t			wanpipe_api_cmd;
	api_tx_hdr_t			*tx_hdr;

	int get_interface_configuration(if_cfg_t *wanif_conf_ptr);
	int GetWANConfig( void );
	unsigned char get_wan_udphdr_data_byte(unsigned char off);
	unsigned char *get_wan_udphdr_data_ptr(unsigned char off);
	unsigned char set_wan_udphdr_data_byte(unsigned char off, unsigned char data);
	int get_fe_type(unsigned char* adapter_type);

	int wanpipe_api_ioctl(wan_cmd_api_t *api_cmd);

	USHORT	DoReadCommand(HANDLE drv, RX_DATA_STRUCT * pRx);
	UCHAR	DoWriteCommand(HANDLE drv, TX_DATA_STRUCT * pTx);
	UCHAR	DoSetIdleTxBufferCommand(HANDLE drv, TX_DATA_STRUCT *pTx);

public:
	//////////////////////////////////////////////////////////////////
	//data
	char					terminate_tx_rx_threads;
	RX_DATA_STRUCT			rx_data;
	callback_functions_t	callback_functions;
	unsigned char			last_hook_state_event;

	char					device_name[DEV_NAME_LEN];	
	if_cfg_t				wanif_conf_struct;

	//////////////////////////////////////////////////////////////////
	//methods
	sangoma_interface(int wanpipe_number, int interface_number);
	~sangoma_interface();

	int DoManagementCommand(HANDLE drv, wan_udp_hdr_t* packet);
#define DO_COMMAND(wan_udp)	DoManagementCommand(hGeneralCommands, &wan_udp);

	int init(callback_functions_t *callback_functions_ptr);
	int run();
	int stop();

	void read_te1_56k_stat(void);
	void set_lb_modes(unsigned char type, unsigned char mode);
	
	void operational_stats (void);
	void flush_operational_stats (void);

	char get_rbs(rbs_management_t *rbs_management_ptr);
	char set_rbs(rbs_management_t *rbs_management_ptr);

	int get_open_handles_counter();

	HANDLE open_api_device(char *api_device_name);

	UCHAR transmit(TX_DATA_STRUCT *pTx)
	{
		return DoWriteCommand(hTx, pTx);
	}

	USHORT receive(RX_DATA_STRUCT *pRx)
	{
		return DoReadCommand(hRx, pRx);
	}

	UCHAR DoSetSharedEventCommand(PREGISTER_EVENT event)
	{
		DWORD ln;
	
		if(DeviceIoControl(
			hRx,
			IoctlSetSharedEvent,
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPVOID)event,
			sizeof(REGISTER_EVENT),
			(LPDWORD)(&ln),
			(LPOVERLAPPED)NULL
						) == FALSE){
			//check messages log
			return 1;
		}else{
			return 0;
		}
	}

	int tdmv_api_ioctl(wanpipe_tdm_api_cmd_t *api_cmd, int cmd);

	int reset_interface_state();

	int start_ring_tone();
	int stop_ring_tone();

	int start_congestion_tone();
	int stop_congestion_tone();

	int start_busy_tone();
	int stop_busy_tone();

	int start_dial_tone();
	int stop_dial_tone();

	int start_ringing_phone();
	int stop_ringing_phone();

	int fxo_go_off_hook();
	int fxo_go_on_hook();

};

#endif
