#ifndef _WANPIPE_LAPB_H
#define _WANPIPE_LAPB_H 


#if !defined(__WINDOWS__)
#include "stddef.h"
#include "wanpipe_abstr.h"
#include "wanpipe_cfg.h"
#include "wanpipe_lapb_iface.h"
#else
 #if defined (__KERNEL__)
  #include "stddef.h"
  #include <wanpipe_includes.h>
  #include <wanpipe_version.h>
  #include <wanpipe_defines.h>
  #include <wanpipe_abstr.h>
  #include <wanpipe_cfg.h>
  #include <wanpipe_lapb_iface.h>
 #endif

#define WIN_Q921	0

#endif



#define	LAPB_HEADER_LEN	20		/* LAPB over Ethernet + a bit more */

#define	LAPB_ACK_PENDING_CONDITION	0x01
#define	LAPB_REJECT_CONDITION		0x02
#define	LAPB_PEER_RX_BUSY_CONDITION	0x04
#define LAPB_RNR_SENT_CONDITION		0x08

#define LAPB_RX_Q_H_LIMIT 4
#define LAPB_RX_Q_L_LIMIT 3

#define LAPB_PROT 	0x00
#define LAPD_PROT	0x01

#define LAPD_SAPI	0x02
#define LAPD_CR_BIT	1

	/* Control field templates */
#define	LAPB_I		0x00	/* Information frames */
#define	LAPB_S		0x01	/* Supervisory frames */
#define	LAPB_U		0x03	/* Unnumbered frames */

#define	LAPB_RR		0x01	/* Receiver ready */
#define	LAPB_RNR	0x05	/* Receiver not ready */
#define	LAPB_REJ	0x09	/* Reject */

#define	LAPB_SABM	0x2F	/* Set Asynchronous Balanced Mode */
#define	LAPB_SABME	0x6F	/* Set Asynchronous Balanced Mode Extended */
#define	LAPB_DISC	0x43	/* Disconnect */
#define	LAPB_DM		0x0F	/* Disconnected mode */
#define	LAPB_UA		0x63	/* Unnumbered acknowledge */
#define	LAPB_FRMR	0x87	/* Frame reject */

#define LAPB_ILLEGAL	0x100	/* Impossible to be a real frame type */
#define LAPB_CORRUPT    0x101   /* Corrupted address field */

#define	LAPB_SPF	0x10	/* Poll/final bit for standard LAPB */
#define	LAPB_EPF	0x01	/* Poll/final bit for extended LAPB */

#define	LAPB_FRMR_W	0x01	/* Control field invalid	*/
#define	LAPB_FRMR_X	0x02	/* I field invalid		*/
#define	LAPB_FRMR_Y	0x04	/* I field too long		*/
#define	LAPB_FRMR_Z	0x08	/* Invalid N(R)			*/

#define	LAPB_POLLOFF	0
#define	LAPB_POLLON	1

	/* LAPB C-bit */
#define LAPB_COMMAND	1
#define LAPB_RESPONSE	2

#define	LAPB_ADDR_A	0x03
#define	LAPB_ADDR_B	0x01
#define	LAPB_ADDR_C	0x0F
#define	LAPB_ADDR_D	0x07

#define TX_TIMEOUT  5
	/* Define Link State constants. */
	enum {
		LAPB_STATE_DISCONNECTED,		/* Disconnected State		*/
		LAPB_STATE_CONNECTING,		/* Awaiting Connection State	*/
		LAPB_STATE_DISCONNECTING, 	/* Awaiting Disconnection State	*/
		LAPB_STATE_CONNECTED,		/* Data Transfer State		*/
		LAPB_STATE_FRAME_REJECT		/* Frame Reject State		*/
	};

#define	LAPB_DEFAULT_MODE		(LAPB_STANDARD | LAPB_SLP | LAPB_DTE)
#define	LAPB_DEFAULT_WINDOW		7		/* Window=7 */
#define	LAPB_DEFAULT_T1			(5 * wpabs_get_hz())	/* T1=5s    */
#define	LAPB_DEFAULT_T2			(1 * wpabs_get_hz())	/* T2=1s    */
#define	LAPB_DEFAULT_N2			20		/* N2=20    */

#define	LAPB_SMODULUS	8
#define	LAPB_EMODULUS	128

#define LAPB_UNREG_CRIT		0
#define LAPB_SMP_CRIT		1
#define LAPB_AWAITING_KICK 	2
#define LAPB_FRAME_SENT		3
#define LAPB_RX_SMP		4
#define LAPB_BH_RUNNING		5
#define LAPB_TIMER_CRIT		6
#define LAPB_REQUEUING		7
#define LAPB_DOWN		8

#define	LAPB_OK			0
#define	LAPB_BADTOKEN		1
#define	LAPB_INVALUE		2
#define	LAPB_CONNECTED		3
#define	LAPB_NOTCONNECTED	4
#define	LAPB_REFUSED		5
#define	LAPB_TIMEDOUT		6
#define	LAPB_NOMEM		7

#define	LAPB_STANDARD		0x00
#define	LAPB_EXTENDED		0x01

#define	LAPB_SLP		0x00
#define	LAPB_MLP		0x02

#define	LAPB_DTE		0x00
#define	LAPB_DCE		0x04

/*
 *	Information about the current frame.
 */
struct lapb_frame {
	unsigned short		type;		/* Parsed type		*/
	unsigned short		nr, ns;		/* N(R), N(S)		*/
	unsigned char		cr;		/* Command/Response	*/
	unsigned char		pf;		/* Poll/Final		*/
	unsigned char		control[2];	/* Original control data*/
};

struct lapb_protocol_stats {
	unsigned int rr_tx;
	unsigned int rnr_tx;
	unsigned int rej_tx;
	unsigned int sabm_tx;
	unsigned int ua_tx;
	unsigned int disc_tx;
	unsigned int dm_tx;
	unsigned int ifrm_tx;
	unsigned int frmr_tx;
	unsigned int ill_tx;
	
	unsigned int rr_rx;
	unsigned int rnr_rx;
	unsigned int rej_rx;
	unsigned int sabm_rx;
	unsigned int ua_rx;
	unsigned int disc_rx;
	unsigned int dm_rx;
	unsigned int ifrm_rx;
	unsigned int frmr_rx;
	unsigned int ill_rx;
};

/*
 *	The per LAPB connection control structure.
 */
typedef struct lapb_cb {
	
	unsigned long		critical;
	
	/* Link status fields */
	wan_lapb_if_conf_t	cfg;

	unsigned int		mode;
	unsigned int		prot;
	unsigned char		state;
	volatile unsigned short	vs, vr, va;
	unsigned char		condition;
	unsigned short		n2, n2count, n4count;
	unsigned short		t1, t2, t3, t3_orig, t4, t4_orig;

	unsigned long		t1timer, t2timer, t4timer;

	/* Internal control information */
	void 			*write_queue;
	void			*ack_queue;
	void			*async_queue;
	void			*rx_queue;
	unsigned char		window;
	unsigned int	 	mtu;
	
	/* FRMR control information */
	struct lapb_frame	frmr_data;
	unsigned char		frmr_type;

	/* Transmit device */
	void 			*dev;
	void 			*chan_dev;

	unsigned char 		*hw_name;

	unsigned char		name[50];

	/* X25 layer interface */
	void * x25_link;
	void * sk_id;
	
	struct lapb_protocol_stats lapbstats;

	unsigned int 		kick_cnt;
	unsigned short 		old_nr,old_nr_type,nr_type;

	wplip_prot_reg_t	reg;

	signed int		refcnt;

} lapb_cb;

/* lapb_iface.c */
extern void lapb_connect_indication(lapb_cb *, int);
extern void lapb_disconnect_indication(lapb_cb *, int);
extern int  lapb_data_indication(lapb_cb * lapb, void *);
extern int  lapb_data_transmit(lapb_cb *, void  *);
extern int  lapb_prot_transmit(lapb_cb *, void  *);

/* lapb_in.c */
extern void lapb_data_input(lapb_cb *, void  *);

/* lapb_out.c */
extern int lapb_bh(lapb_cb*);
extern int lapb_transmit_buffer(lapb_cb *, void  *, int,char);
extern void lapb_establish_data_link(lapb_cb *);
extern void lapb_enquiry_response(lapb_cb *);
extern void lapb_timeout_response(lapb_cb *);
extern inline void lapb_check_iframes_acked(lapb_cb *, unsigned short);
extern void lapb_check_need_response(lapb_cb *, int, int);

/* lapb_subr.c */
extern lapb_cb* lapb_create_cb(void);
extern int 	lapb_free_cb(lapb_cb*);
extern void 	lapb_requeue_frames(lapb_cb *);
extern void 	lapb_decode(lapb_cb *, void  *, struct lapb_frame *);
extern void 	lapb_send_control(lapb_cb *, char, int, int);
extern void 	lapb_transmit_frmr(lapb_cb *);
extern void 	lapb_update_stats (lapb_cb *lapb, int type, int len, char direction);
extern void 	lapb_state_change(lapb_cb *lapb, unsigned char state);
extern void 	lapb_reconnect(lapb_cb*lapb);
extern void 	lapb_link_up (lapb_cb *lapb);
extern void 	lapb_link_down (lapb_cb *lapb);


/* lapb_timer.c */
extern void lapb_timer (void *);
extern void lapb_start_t1timer(lapb_cb *);
extern void lapb_start_t2timer(lapb_cb *);
extern void lapb_start_t4timer(lapb_cb *, unsigned char rest_t4_count);
extern void lapb_stop_t1timer(lapb_cb *);
extern void lapb_stop_t2timer(lapb_cb *);
extern void lapb_stop_t4timer(lapb_cb *);
extern int  lapb_t1timer_running(lapb_cb *);
extern void lapb_t1timer_expiry(lapb_cb *lapb);
extern void lapb_t4timer_expiry(lapb_cb *lapb);
extern void lapb_t2timer_expiry(lapb_cb *lapb);

#define wp_dev_hold(dev)    do{\
				wpabs_debug_test("%s:%d (dev_hold) Refnt %i!\n",\
							__FUNCTION__,__LINE__,dev->refcnt );\
				dev->refcnt++; \
			    }while(0)

#define wp_dev_put(dev)	    do{\
				wpabs_debug_test("%s:%d (wp_dev_put): Refnt %i!\n",\
							__FUNCTION__,__LINE__,dev->refcnt );\
				if (--dev->refcnt == 0){\
					wpabs_debug_test("%s:%d (wp_dev_put): Deallocating!\n",\
							__FUNCTION__,__LINE__);\
					wpabs_free(dev);\
					dev=NULL;\
				}\
		            }while(0)

#if defined(__WINDOWS__)

#define FUNC_BEGIN()	if(0)DEBUG_LIP("%s:%d ---Begin---\n",__FUNCTION__,__LINE__); 
#define FUNC_END()		if(0)DEBUG_LIP("%s:%d ---End---\n\n",__FUNCTION__,__LINE__); 

#define FUNC_BEGIN1()	if(0)DEBUG_LIP("%s:%d ---Begin---\n",__FUNCTION__,__LINE__); 
#define FUNC_END1()		if(0)DEBUG_LIP("%s:%d ---End---\n\n",__FUNCTION__,__LINE__); 

#else

#define FUNC_BEGIN() wpabs_debug_test("%s:%d ---Begin---\n",__FUNCTION__,__LINE__); 
#define FUNC_END() wpabs_debug_test("%s:%d ---End---\n\n",__FUNCTION__,__LINE__); 

#define FUNC_BEGIN1() wpabs_debug_event("%s:%d ---Begin---\n",__FUNCTION__,__LINE__); 
#define FUNC_END1() wpabs_debug_event("%s:%d ---End---\n\n",__FUNCTION__,__LINE__); 

#endif


/*
 *	This routine purges all the queues of frames.
 */

static __inline void lapb_clear_queues(lapb_cb *lapb)
{
	FUNC_BEGIN();
	wpabs_skb_queue_purge(lapb->write_queue);
	wpabs_skb_queue_purge(lapb->ack_queue);
	wpabs_skb_queue_purge(lapb->async_queue);
	wpabs_skb_queue_purge(lapb->rx_queue);
}

/*
 *	Validate that the value of nr is between va and vs. Return true or
 *	false for testing.
 */
static __inline int lapb_validate_nr(lapb_cb *lapb, volatile unsigned short nr)
{
	volatile unsigned short vc = lapb->va;
	volatile int modulus;
	
	modulus = (lapb->mode & LAPB_EXTENDED) ? LAPB_EMODULUS : LAPB_SMODULUS;

	while (vc != lapb->vs) {
		if (nr == vc) {
			lapb->old_nr=nr;
			return 1;
		}
		vc = (vc + 1) % modulus;
	}
	
	if (nr == lapb->vs) {
		lapb->old_nr=nr;
		return 1;
	}
	wpabs_debug_event( "%s: Error : Rx Frame=%i Invalid NR=%i, PrevNR=%i PrevType=%i Vs=%i, Va=%i\n",
			lapb->name, lapb->nr_type, nr, lapb->old_nr,
			lapb->old_nr_type,lapb->vs,lapb->va);
	
	return 0;
}


/*
 * This routine purges the input queue of those frames that have been
 * acknowledged. This replaces the boxes labelled "V(a) <- N(r)" on the
 * SDL diagram.
 */
#define lapb_frames_acked(x, y)	dbg_lapb_frames_acked(x, y, __FUNCTION__, __LINE__)

static __inline void dbg_lapb_frames_acked(lapb_cb *lapb, unsigned short nr,
									   char *called_from_func, int line)
{
	void *skb;
	volatile int modulus;

	DEBUG_LIP("%s():from:%s():line:%d\n", __FUNCTION__, called_from_func, line);

	modulus = (lapb->mode & LAPB_EXTENDED) ? LAPB_EMODULUS : LAPB_SMODULUS;

	/*
	 * Remove all the ack-ed frames from the ack queue.
	 */

	if (lapb->va != nr) {
		while (wpabs_skb_queue_len(lapb->ack_queue) != 0 && lapb->va != nr) {
	        skb = wpabs_skb_dequeue(lapb->ack_queue);
			lapb->va = (lapb->va + 1) % modulus;
			wpabs_skb_free(skb);
		}
	}
}


#if 0
static __inline void lapb_frames_acked(lapb_cb *lapb, unsigned short nr)
{
	void *skb;
	volatile int modulus;

	modulus = (lapb->mode & LAPB_EXTENDED) ? LAPB_EMODULUS : LAPB_SMODULUS;

	/*
	 * Remove all the ack-ed frames from the ack queue.
	 */

	if (lapb->va != nr) {
		while (wpabs_skb_queue_len(lapb->ack_queue) != 0 && lapb->va != nr) {
	        skb = wpabs_skb_dequeue(lapb->ack_queue);
			lapb->va = (lapb->va + 1) % modulus;
			wpabs_skb_free(skb);
		}
	}
}
#endif

#endif
