/*==================================================================
 Sample program for the "LibSangoma API".

 Copyright (c) 1984-2007 Sangoma Technologies Inc.

 Target: Windows XP/2003 and later. Win32 Console.

 For Version beta-6,0,0,2 or later of the API driver (sdladrv.sys).

==================================================================*/

//Note 1: the header files are in "include" directory,
//		your compiler should know where to look for
//		the Header files.
//
//Note 2: the API output is "BitSwapped", the input is expected
//		to be "BitSwapped" too. Please use code in "api\bit_swap" 
//		subdirectory as an example of very efficient "bitswapping"
//		algorithm for a voice stream.

#define __WINDOWS__

#include <libsangoma.h>		//for Wanpipe API

#include "lib_api.h"		
//////////////////////////////////////////////////////////////////

#define MAX_TX_DATA		6000	/* Size of tx data */  
#define MAX_FRAMES 		6000    /* Number of frames to transmit */  
#define MAX_RX_DATA		6000


u_int32_t	poll_events_bitmap = 0;	

unsigned char Rx_data[MAX_RX_DATA + sizeof(wp_tdm_api_rx_hdr_t)];
unsigned char Tx_data[MAX_TX_DATA + sizeof(wp_tdm_api_rx_hdr_t)];

HANDLE dev_fd = INVALID_HANDLE_VALUE;
wanpipe_tdm_api_t tdm_api; 

unsigned char rx_rbs_bits = WAN_RBS_SIG_A;

//////////////////////////////////////////////////////////////////
BOOL 
WINAPI 
TerminateHandler(
	DWORD dwCtrlType
	);

void 
cleanup(
	void
	);

void
handle_span_chan(
	void
	);

int 
dtmf_event(
	sng_fd_t fd, 
	unsigned char digit, 
	unsigned char type,
	unsigned char port
	);

int 
rbs_event(
	sng_fd_t fd, 
	unsigned char rbs_bits
	);

int 
rxhook_event(
	sng_fd_t fd, 
	unsigned char hook_state
	);

int 
rxring_event(
	sng_fd_t fd, 
	unsigned char ring_state
	);

int 
rxring_trip(
	sng_fd_t fd, 
	unsigned char ring_state
	);

void
print_rx_data(
	wp_tdm_api_rx_element_t *api_rx_el,
	int	datalen
	);

//////////////////////////////////////////////////////////////////

void
print_rx_data(
	wp_tdm_api_rx_element_t *api_rx_el,
	int	datalen
	)
{
	int i;

	printf("Data: ");
	for(i = 0; i < datalen; i++) {
		printf("0x%02X ", api_rx_el->data[i]);
		//don't print too much
		if(i && !(i % 30)){
			printf("...\n");
			break;
		}
	}
	printf("\n");
}

void 
handle_span_chan(
	void
	) 
{
	int Rx_count,Tx_count;
	int Rx_lgth;
	unsigned short Tx_length;

	const wp_tdm_api_tx_element_t *api_tx_el = (wp_tdm_api_tx_element_t*)&Tx_data[0];
	const wp_tdm_api_rx_element_t *api_rx_el = (wp_tdm_api_rx_element_t*)&Rx_data[0];

	const wp_tdm_api_rx_hdr_t *tdm_api_rx_hdr = (wp_tdm_api_rx_hdr_t*)&Rx_data[0];
	const wp_tdm_api_rx_hdr_t *tdm_event      = (wp_tdm_api_rx_hdr_t*)&Rx_data[sizeof(wp_tdm_api_rx_hdr_t)];

	int api_poll_status;
	int	err;
	unsigned char tx_test_byte=0;	
	
	Rx_count = 0;
	Tx_count = 0;

	printf("\n\nSpan/Chan Handler: RxEnable=%s, TxEnable=%s, TxCnt=%i, TxLen=%i\n",
		(read_enable? "Yes":"No"), (write_enable?"Yes":"No"),tx_cnt,tx_size);	

	/* Initialize the Tx Data buffer */
	memset(&Tx_data[0],0,MAX_TX_DATA + sizeof(wp_tdm_api_rx_hdr_t));

	/* Main Rx/Tx/Event loop */
	for(;;) 
	{	
		api_poll_status = sangoma_socket_waitfor(
								dev_fd, 
								1000,		//timeout in Milliseconds
								poll_events_bitmap);
		if(!api_poll_status){
			printf("SAMPLE: Error: sangoma_socket_waitfor() failed!\n");
			printf("\tMake sure another Process/Thread is NOT using the SAME 'span/chan',\n");
			printf("\tand interface is configured in 'LIB SANGOMA API' mode.\n");
			break;
		}

		if(api_poll_status & POLL_EVENT_TIMEOUT){
			//poll again
			print_poll_event_bitmap(api_poll_status);
			continue;
		}
		if(api_poll_status & POLL_EVENT_LINK_DISCONNECT){
			//line disconnected, wait for it to re-connect, poll again.
			print_poll_event_bitmap(api_poll_status);
			printf("\n");
			continue;
		}
		if(api_poll_status & POLL_EVENT_LINK_CONNECT){
			//line connected, there is a chance for RX data.
			print_poll_event_bitmap(api_poll_status);
			printf("\n");
		}

		if(api_poll_status & POLL_EVENT_RX_DATA){
			//read the message
			Rx_lgth = sangoma_readmsg_tdm(
						dev_fd,
						Rx_data, 
						sizeof(wp_tdm_api_rx_hdr_t),
						&Rx_data[sizeof(wp_tdm_api_rx_hdr_t)],
						MAX_RX_DATA, 0);   
			if(Rx_lgth <= 0) {
				printf("Error receiving data\n\n");
				break;
			}

			switch(tdm_api_rx_hdr->wp_tdm_api_event_type)
			{
			case SANG_STATUS_RX_DATA_AVAILABLE:
				Rx_count++;
				if((Rx_count % 100) == 0){
					printf("Receive Counter: %i, Data Length: %i\n", Rx_count, Rx_lgth);
				}
				if (verbose){
					print_rx_data((wp_tdm_api_rx_element_t *)api_rx_el, Rx_lgth);
				}

				//use Rx_counter as "write" events trigger:
				if(rbs_events == 1 && (Rx_count % 400) == 0){
					/*	bitmap - set as needed: WAN_RBS_SIG_A |	WAN_RBS_SIG_B | WAN_RBS_SIG_C | WAN_RBS_SIG_D;

					In this example make bits A and B to change each time,
					so it's easy to see the change on the receiving side.
					*/
					if(rx_rbs_bits == WAN_RBS_SIG_A){
						rx_rbs_bits = WAN_RBS_SIG_B;
					}else{
						rx_rbs_bits = WAN_RBS_SIG_A;
					}
					printf("Writing RBS bits (0x%X)...\n", rx_rbs_bits);
					sangoma_tdm_write_rbs(dev_fd, &tdm_api, rx_rbs_bits);
				}
				break;

			case SANG_STATUS_TDM_EVENT_AVAILABLE:
				sangoma_tdm_read_event(dev_fd, &tdm_api);
				break;

			default:
				printf("Invalid API_EVENT_TYPE: 0x%x!!!\n", tdm_api_rx_hdr->wp_tdm_api_event_type);
				break;
			}//switch()
		}//if()

		if(api_poll_status & POLL_EVENT_TX_READY && write_enable){
			//
			//Note: 1.  if NO codec is used, transmit length should be EQUAL to interface MTU!
			//		2.  if a codec is used (Voice channels only), compression rate should be 
			//			taken in consideration, so actual transmit length, AFTER the compression,
			//			is EQUAL to inerface MTU!
			//
			if(	api_poll_status & POLL_EVENT_RX_DATA &&
				tdm_api_rx_hdr->wp_tdm_api_event_type == SANG_STATUS_RX_DATA_AVAILABLE){

				//Send back received data ("software loopback"), just a test.
				Tx_length = Rx_lgth;
				memcpy(&Tx_data[16], api_rx_el->data, Tx_length);
			}else{
				Tx_length = 128;//128 is just an example
				memset(&Tx_data[16], tx_test_byte, Tx_length);
				tx_test_byte++;
			}

			//write a message
			err = sangoma_writemsg_tdm(
						dev_fd,
						Tx_data,16, 
						&Tx_data[16], Tx_length, 
						0);
			if (err <= 0){
				printf("Failed to send!\n");
				break;
			}else{
				Tx_count++;
				if (verbose){
					printf("Packet sent: counter: %i, len: %i\n", Tx_count, err);
				}else{
					printf("T\n");
				}
			}

			if(Tx_count >= tx_cnt){
				write_enable=0;
				printf("Disabling POLL_EVENT_TX_READY...\n");
				//
				//No need for POLL_EVENT_TX_READY, turn it off!! If not turned off, and we
				//have nothing for transmission, sangoma_socket_waitfor() will return
				//immediately, creating a busy loop.
				//
				poll_events_bitmap &= (~POLL_EVENT_TX_READY);
			}
		}//if()
	}//for()
}

int dtmf_event (sng_fd_t fd, unsigned char digit, unsigned char type, unsigned char port)
{
	printf("DTMF Event: Digit: %c (Port: %s, Type:%s)!\n",
			digit,
			(port == WAN_EC_CHANNEL_PORT_ROUT)?"ROUT":"SOUT",
			(type == WAN_EC_TONE_PRESENT)?"PRESENT":"STOP");		
	return 0;
}

int rbs_event (sng_fd_t fd, unsigned char rbs_bits)
{
	printf("RBS Event: BITS=0x%X, A:%1d B:%1d C:%1d D:%1d\n",
			rbs_bits, 
			(rbs_bits & WAN_RBS_SIG_A) ? 1 : 0,
			(rbs_bits & WAN_RBS_SIG_B) ? 1 : 0,
			(rbs_bits & WAN_RBS_SIG_C) ? 1 : 0,
			(rbs_bits & WAN_RBS_SIG_D) ? 1 : 0);

	rx_rbs_bits = rbs_bits;
	return 0;
}

int rxhook_event (sng_fd_t fd, unsigned char hook_state)
{
	printf("rxhook_event(): %s (0x%X)\n", 
		WAN_EVENT_RXHOOK_DECODE(hook_state), hook_state);
	return 0;
}

int rxring_event (sng_fd_t fd, unsigned char ring_state)
{
	printf("rxring_event(): %s (0x%X)\n",
		WAN_EVENT_RING_DECODE(ring_state), ring_state);
	return 0;
}

int ringtrip_event (sng_fd_t fd, unsigned char ring_state)
{
	printf("ringtrip_event(): %s (0x%X)\n", 
		WAN_EVENT_RING_TRIP_DECODE(ring_state), ring_state);
	return 0;
}

/*
 * TerminateHandler() - this handler is called by the system whenever user tries to terminate
 *						the process with Ctrl+C, Ctrl+Break or closes the console window.
 *						Perform a clean-up here.
 */
BOOL WINAPI 
TerminateHandler(DWORD dwCtrlType)
{
	printf("\nProcess terminated by user request.\n");

	//do the cleanup before exiting:
	cleanup();

	//return FALSE so the system will call the dafult handler which will terminate the process.
	return FALSE;
}

void 
cleanup()
{
	printf("cleanup()...\n");

	if(dtmf_enable_octasic == 1){
		/* Disable dtmf detection on Octasic chip */
		sangoma_tdm_disable_dtmf_events(dev_fd, &tdm_api);
	}

	if(dtmf_enable_remora == 1){
		/* Disable dtmf detection on Sangoma's Remora SLIC chip */
		sangoma_tdm_disable_rm_dtmf_events(dev_fd, &tdm_api);
	}

	if(remora_hook == 1){
		sangoma_tdm_disable_rxhook_events(dev_fd, &tdm_api);
	}

	if(rbs_events == 1){
    	sangoma_tdm_disable_rbs_events(dev_fd, &tdm_api);
	}

	//call sangoma_socket_close() for EACH open Device Handle
	sangoma_socket_close(&dev_fd);
}

int main(int argc, char* argv[])
{
	int proceed, span, chan;

	proceed=init_args(argc,argv);
	if (proceed != WAN_TRUE){
		usage(argv[0]);
		return -1;
	}

	/* register Ctrl+C handler - we want a clean termination */
	if (!SetConsoleCtrlHandler(TerminateHandler, TRUE)) {
		printf("ERROR : Unable to register terminate handler ( %d ).\nProcess terminated.\n", 
			GetLastError());
		return -1;
	}

	poll_events_bitmap = 0;
	if(read_enable	== 1){
		poll_events_bitmap |= POLLIN;
	}
	
	if(write_enable == 1){
		poll_events_bitmap |= POLLOUT;
	}

	/* always need to know about Front End connect/disconnect */
	poll_events_bitmap |= POLLHUP;

	printf("Enabling Poll Events:\n");
	print_poll_event_bitmap(poll_events_bitmap);

	if (card_cnt){
		printf("Using Card name: %s, Interface name: %s\n", card_name, if_name);
	}else{
		printf("Using Card/Interface name: %s\n", if_name);
	}

	memset(&tdm_api,0,sizeof(tdm_api));
	tdm_api.wp_tdm_event.wp_dtmf_event = &dtmf_event;
	tdm_api.wp_tdm_event.wp_rbs_event = &rbs_event;
	tdm_api.wp_tdm_event.wp_rxhook_event = &rxhook_event;
	tdm_api.wp_tdm_event.wp_rxring_event = &rxring_event;
	tdm_api.wp_tdm_event.wp_ringtrip_event = &ringtrip_event;


	printf("TDM DTMF PTR = %p\n",tdm_api.wp_tdm_event.wp_dtmf_event);

	if (!interface_cnt){
		/* Find and open 1-st UNUSED channel on the span: */
		dev_fd = sangoma_open_tdmapi_span(atoi(card_name));
	}else if(!card_cnt){
		/* try to parse "w?g?" type of name (i.g.: w1g1): */
		if(sangoma_interface_toi(if_name, &span, &chan) == 0){
			/*failed to parse.
			may be it is "s?c?" type of name (i.g.: s1c1), try to parse it:	*/
			if(sangoma_span_chan_fromif(if_name, &span, &chan) == 0){
				/* failed too. */
				printf("Failed to parse Interface Name: %s!!\n", if_name);
				return -1;
			}
		}
		dev_fd = sangoma_open_tdmapi_span_chan(span, chan);
	}else{
		/* open a known Span/Chan combination: */
		dev_fd = sangoma_open_tdmapi_span_chan(atoi(card_name),atoi(if_name));
	}
	if( dev_fd == INVALID_HANDLE_VALUE){
		printf("Failed to open span chan\n");
		return -1;
	}

	printf("HANDLING SPAN %i CHAN %i\n", atoi(card_name), atoi(if_name));

	sangoma_get_full_cfg(dev_fd, &tdm_api);

	if(set_codec_slinear){
		printf("Setting SLINEAR codec\n");
		sangoma_tdm_set_codec(dev_fd, &tdm_api, WP_SLINEAR);
	}

	if(set_codec_none){
		printf("Disabling codec\n");
		sangoma_tdm_set_codec(dev_fd, &tdm_api, WP_NONE);
	}

	if(usr_period){
		printf("Setting user period: %d\n", usr_period);
		sangoma_tdm_set_usr_period(dev_fd, &tdm_api, usr_period);
	}

	if(set_codec_slinear || usr_period || set_codec_none){
		//display new configuration AFTER it was changed
		sangoma_get_full_cfg(dev_fd, &tdm_api);
	}

	if(dtmf_enable_octasic == 1){
		/* enable dtmf detection on Octasic chip */
		sangoma_tdm_enable_dtmf_events(dev_fd, &tdm_api);
	}

	if(dtmf_enable_remora == 1){
		/* enable dtmf detection on Sangoma's Remora SLIC chip (A200 ONLY) */
		sangoma_tdm_enable_rm_dtmf_events(dev_fd, &tdm_api);
	}

	if(remora_hook == 1){
		sangoma_tdm_enable_rxhook_events(dev_fd, &tdm_api);
	}

	if(rbs_events == 1){
    	sangoma_tdm_enable_rbs_events(dev_fd, &tdm_api, 20);
	}

	Sleep(1000);//wait for a second, so user can read the output

	handle_span_chan();

	//returned from main loop, do the cleanup before exiting:
	cleanup();

	printf("\nSample application exiting.(press any key)\n");
	_getch();
	return 0;
}

 


